import AppKit
import objc
from PyObjCTools.TestSupport import TestCase


class TestNSInputServerHelper(AppKit.NSObject):
    def doCommandBySelector_client_(self, sel, sender):
        pass

    def markedTextSelectionChanged_client_(self, sel, sender):
        pass

    def canBeDisabled(self):
        return 1

    def wantsToInterpretAllKeystrokes(self):
        return 1

    def wantsToHandleMouseEvents(self):
        return 1

    def wantsToDelayTextChangeNotifications(self):
        return 1

    def activeConversationWillChange_fromOldConversation_(self, s, o):
        pass

    def activeConversationChanged_toNewConversation_(self, s, n):
        pass

    def mouseDownOnCharacterIndex_atCoordinate_withModifier_client_(self, i, p, f, s):
        return 1

    def mouseDraggedOnCharacterIndex_atCoordinate_withModifier_client_(
        self, i, p, f, s
    ):
        return 1

    def mouseUpOnCharacterIndex_atCoordinate_withModifier_client_(self, i, p, f, s):
        pass


class TestNSInputServer(TestCase):
    def testProtocols(self):
        objc.protocolNamed("NSInputServiceProvider")
        self.assertArgIsSEL(
            TestNSInputServerHelper.doCommandBySelector_client_, 0, b"v@:@"
        )
        self.assertArgHasType(
            TestNSInputServerHelper.markedTextSelectionChanged_client_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultIsBOOL(TestNSInputServerHelper.canBeDisabled)
        self.assertResultIsBOOL(TestNSInputServerHelper.wantsToInterpretAllKeystrokes)
        self.assertResultIsBOOL(TestNSInputServerHelper.wantsToHandleMouseEvents)
        self.assertResultIsBOOL(
            TestNSInputServerHelper.wantsToDelayTextChangeNotifications
        )
        self.assertArgHasType(
            TestNSInputServerHelper.activeConversationWillChange_fromOldConversation_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSInputServerHelper.activeConversationChanged_toNewConversation_,
            1,
            objc._C_NSInteger,
        )

        objc.protocolNamed("NSInputServerMouseTracker")

        self.assertResultIsBOOL(
            TestNSInputServerHelper.mouseDownOnCharacterIndex_atCoordinate_withModifier_client_
        )
        self.assertArgHasType(
            TestNSInputServerHelper.mouseDownOnCharacterIndex_atCoordinate_withModifier_client_,
            0,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSInputServerHelper.mouseDownOnCharacterIndex_atCoordinate_withModifier_client_,
            1,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSInputServerHelper.mouseDownOnCharacterIndex_atCoordinate_withModifier_client_,
            2,
            objc._C_NSUInteger,
        )

        self.assertResultIsBOOL(
            TestNSInputServerHelper.mouseDraggedOnCharacterIndex_atCoordinate_withModifier_client_  # noqa: B950
        )
        self.assertArgHasType(
            TestNSInputServerHelper.mouseDraggedOnCharacterIndex_atCoordinate_withModifier_client_,  # noqa: B950
            0,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSInputServerHelper.mouseDownOnCharacterIndex_atCoordinate_withModifier_client_,
            1,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSInputServerHelper.mouseDownOnCharacterIndex_atCoordinate_withModifier_client_,
            2,
            objc._C_NSUInteger,
        )

        self.assertResultHasType(
            TestNSInputServerHelper.mouseUpOnCharacterIndex_atCoordinate_withModifier_client_,
            b"v",
        )
        self.assertArgHasType(
            TestNSInputServerHelper.mouseUpOnCharacterIndex_atCoordinate_withModifier_client_,
            0,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSInputServerHelper.mouseUpOnCharacterIndex_atCoordinate_withModifier_client_,
            1,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSInputServerHelper.mouseUpOnCharacterIndex_atCoordinate_withModifier_client_,
            2,
            objc._C_NSUInteger,
        )



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

setup(
    name="WebServices Tool (CoreData)",
    app=["Main.py"],
    data_files=["English.lproj", "Preferences.png", "Reload.png", "WST.png"],
    options={"py2app": {"iconfile": "WST.icns"}},
    setup_requires=["py2app", "pyobjc-framework-Cocoa"],
)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class FoundationErrorsTest(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSFileNoSuchFileError, 4)
        self.assertEqual(Foundation.NSFileLockingError, 255)
        self.assertEqual(Foundation.NSFileReadUnknownError, 256)
        self.assertEqual(Foundation.NSFileReadNoPermissionError, 257)
        self.assertEqual(Foundation.NSFileReadInvalidFileNameError, 258)
        self.assertEqual(Foundation.NSFileReadCorruptFileError, 259)
        self.assertEqual(Foundation.NSFileReadInapplicableStringEncodingError, 261)
        self.assertEqual(Foundation.NSFileReadUnsupportedSchemeError, 262)
        self.assertEqual(Foundation.NSFileReadTooLargeError, 263)
        self.assertEqual(Foundation.NSFileReadUnknownStringEncodingError, 264)
        self.assertEqual(Foundation.NSFileWriteUnknownError, 512)
        self.assertEqual(Foundation.NSFileWriteNoPermissionError, 513)
        self.assertEqual(Foundation.NSFileWriteInvalidFileNameError, 514)
        self.assertEqual(Foundation.NSFileWriteInapplicableStringEncodingError, 517)
        self.assertEqual(Foundation.NSFileWriteUnsupportedSchemeError, 518)
        self.assertEqual(Foundation.NSFileWriteOutOfSpaceError, 640)
        self.assertEqual(Foundation.NSKeyValueValidationError, 1024)
        self.assertEqual(Foundation.NSUserCancelledError, 3072)
        self.assertEqual(Foundation.NSExecutableNotLoadableError, 3584)
        self.assertEqual(Foundation.NSExecutableArchitectureMismatchError, 3585)
        self.assertEqual(Foundation.NSExecutableRuntimeMismatchError, 3586)
        self.assertEqual(Foundation.NSExecutableLoadError, 3587)
        self.assertEqual(Foundation.NSExecutableLinkError, 3588)
        self.assertEqual(Foundation.NSFileErrorMinimum, 0)
        self.assertEqual(Foundation.NSFileErrorMaximum, 1023)
        self.assertEqual(Foundation.NSValidationErrorMinimum, 1024)
        self.assertEqual(Foundation.NSValidationErrorMaximum, 2047)
        self.assertEqual(Foundation.NSExecutableErrorMinimum, 3584)
        self.assertEqual(Foundation.NSExecutableErrorMaximum, 3839)
        self.assertEqual(Foundation.NSFormattingErrorMinimum, 2048)
        self.assertEqual(Foundation.NSFormattingErrorMaximum, 2559)
        self.assertEqual(Foundation.NSFormattingError, 2048)
        self.assertEqual(Foundation.NSCoderInvalidValueError, 4866)

        self.assertEqual(Foundation.NSBundleOnDemandResourceOutOfSpaceError, 4992)
        self.assertEqual(
            Foundation.NSBundleOnDemandResourceExceededMaximumSizeError, 4993
        )
        self.assertEqual(Foundation.NSBundleOnDemandResourceInvalidTagError, 4994)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(Foundation.NSFileWriteVolumeReadOnlyError, 642)

        self.assertEqual(Foundation.NSPropertyListReadCorruptError, 3840)
        self.assertEqual(Foundation.NSPropertyListReadUnknownVersionError, 3841)
        self.assertEqual(Foundation.NSPropertyListReadStreamError, 3842)
        self.assertEqual(Foundation.NSPropertyListWriteStreamError, 3851)
        self.assertEqual(Foundation.NSPropertyListErrorMinimum, 3840)
        self.assertEqual(Foundation.NSPropertyListErrorMaximum, 4095)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(Foundation.NSFileWriteFileExistsError, 516)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(Foundation.NSFeatureUnsupportedError, 3328)
        self.assertEqual(Foundation.NSXPCConnectionInterrupted, 4097)
        self.assertEqual(Foundation.NSXPCConnectionInvalid, 4099)
        self.assertEqual(Foundation.NSXPCConnectionReplyInvalid, 4101)
        self.assertEqual(Foundation.NSXPCConnectionErrorMinimum, 4096)
        self.assertEqual(Foundation.NSXPCConnectionErrorMaximum, 4224)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertEqual(Foundation.NSUbiquitousFileUnavailableError, 4353)
        self.assertEqual(Foundation.NSUbiquitousFileNotUploadedDueToQuotaError, 4354)
        self.assertEqual(Foundation.NSUbiquitousFileUbiquityServerNotAvailable, 4355)
        self.assertEqual(Foundation.NSUbiquitousFileErrorMinimum, 4352)
        self.assertEqual(Foundation.NSUbiquitousFileErrorMaximum, 4607)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(Foundation.NSUserActivityHandoffFailedError, 4608)
        self.assertEqual(Foundation.NSUserActivityConnectionUnavailableError, 4609)
        self.assertEqual(Foundation.NSUserActivityRemoteApplicationTimedOutError, 4610)
        self.assertEqual(Foundation.NSUserActivityHandoffUserInfoTooLargeError, 4611)
        self.assertEqual(Foundation.NSUserActivityErrorMinimum, 4608)
        self.assertEqual(Foundation.NSUserActivityErrorMaximum, 4863)
        self.assertEqual(Foundation.NSPropertyListWriteInvalidError, 3852)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(Foundation.NSFileManagerUnmountUnknownError, 768)
        self.assertEqual(Foundation.NSFileManagerUnmountBusyError, 769)

        self.assertEqual(Foundation.NSCoderReadCorruptError, 4864)
        self.assertEqual(Foundation.NSCoderValueNotFoundError, 4865)
        self.assertEqual(Foundation.NSCoderErrorMinimum, 4864)
        self.assertEqual(Foundation.NSCoderErrorMaximum, 4991)

        self.assertEqual(Foundation.NSBundleErrorMinimum, 4992)
        self.assertEqual(Foundation.NSBundleErrorMaximum, 5119)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertEqual(Foundation.NSCloudSharingNetworkFailureError, 5120)
        self.assertEqual(Foundation.NSCloudSharingQuotaExceededError, 5121)
        self.assertEqual(Foundation.NSCloudSharingTooManyParticipantsError, 5122)
        self.assertEqual(Foundation.NSCloudSharingConflictError, 5123)
        self.assertEqual(Foundation.NSCloudSharingNoPermissionError, 5124)
        self.assertEqual(Foundation.NSCloudSharingOtherError, 5375)
        self.assertEqual(Foundation.NSCloudSharingErrorMinimum, 5120)
        self.assertEqual(Foundation.NSCloudSharingErrorMaximum, 5375)

    @min_os_level("10.15")
    def testConstants10_15(self):
        self.assertEqual(Foundation.NSCompressionFailedError, 5376)
        self.assertEqual(Foundation.NSDecompressionFailedError, 5377)
        self.assertEqual(Foundation.NSCompressionErrorMinimum, 5376)
        self.assertEqual(Foundation.NSCompressionErrorMaximum, 5503)



"""DotView.py -- A one-window app demonstrating how to write a custom NSView.

To build the demo program, run this line in Terminal.app:

    $ python setup.py py2app -A

This creates a directory "dist" containing DotView.app. (The
-A option causes the files to be symlinked to the .app bundle instead
of copied. This means you don't have to rebuild the app if you edit the
sources or nibs.)
"""

# Created by Etienne Posthumus on Thu Dec 26 2002, after Apple's
# ObjC DotView example.
# Edited and enhanced by JvR, 2003.
#
# The main difference with the Apple DotView demo is that our custom view
# is embedded in a scroll view. It turns out that this is almost no work
# in InterfaceBuilder (select the view, then go to Layout -> Make subvies of
# -> Scroll View), and *no* work in the code. It was too easy, so for kicks
# I added zoom functionality and a "Show rulers" checkbox.

import Cocoa
import objc
from objc import super
from PyObjCTools import AppHelper

ZOOM = 2.0


# class defined in MainMenu.nib
class DotView(Cocoa.NSView):
    colorWell = objc.IBOutlet()
    sizeSlider = objc.IBOutlet()

    def initWithFrame_(self, frame):
        self.center = (50.0, 50.0)
        super().initWithFrame_(frame)
        self.radius = 10.0
        self.color = Cocoa.NSColor.redColor()
        return self

    def awakeFromNib(self):
        self.colorWell.setColor_(self.color)
        self.sizeSlider.setFloatValue_(self.radius)
        scrollView = self.superview().superview()
        scrollView.setHasHorizontalRuler_(1)
        scrollView.setHasVerticalRuler_(1)

    @objc.IBAction
    def zoomIn_(self, sender):
        (x, y), (bw, bh) = self.bounds()
        (x, y), (fw, fh) = self.frame()
        self.setBoundsSize_((bw / ZOOM, bh / ZOOM))
        self.setFrameSize_((fw * ZOOM, fh * ZOOM))
        self.setNeedsDisplay_(True)

    @objc.IBAction
    def zoomOut_(self, sender):
        (x, y), (bw, bh) = self.bounds()
        (x, y), (fw, fh) = self.frame()
        self.setBoundsSize_((bw * ZOOM, bh * ZOOM))
        self.setFrameSize_((fw / ZOOM, fh / ZOOM))
        self.setNeedsDisplay_(True)

    @objc.IBAction
    def setRulersVisible_(self, button):
        scrollView = self.superview().superview()
        scrollView.setRulersVisible_(button.state())

    def isOpaque(self):
        return True

    def mouseDown_(self, event):
        eventLocation = event.locationInWindow()
        if event.modifierFlags() & Cocoa.NSCommandKeyMask:
            clipView = self.superview()
            self.originalPoint = eventLocation
            self.originalOffset = clipView.bounds()[0]
        else:
            self.center = self.convertPoint_fromView_(eventLocation, None)
            self.setNeedsDisplay_(True)
            self.autoscroll_(event)

    def mouseDragged_(self, event):
        if event.modifierFlags() & Cocoa.NSCommandKeyMask:
            clipView = self.superview()
            eventLocation = event.locationInWindow()
            ox, oy = self.originalPoint
            x, y = eventLocation
            dx, dy = x - ox, y - oy
            x, y = self.originalOffset
            ox, oy = clipView.constrainScrollPoint_((x - dx, y - dy))
            clipView.scrollToPoint_((ox, oy))
            clipView.superview().reflectScrolledClipView_(clipView)
        else:
            self.mouseDown_(event)

    def drawRect_(self, rect):
        Cocoa.NSColor.whiteColor().set()
        Cocoa.NSRectFill(self.bounds())
        origin = (self.center[0] - self.radius, self.center[1] - self.radius)
        size = (2 * self.radius, 2 * self.radius)
        dotRect = (origin, size)
        self.color.set()
        Cocoa.NSBezierPath.bezierPathWithOvalInRect_(dotRect).fill()

    @objc.IBAction
    def setRadius_(self, sender):
        self.radius = sender.floatValue()
        self.setNeedsDisplay_(True)

    @objc.IBAction
    def setColor_(self, sender):
        self.color = sender.color()
        self.setNeedsDisplay_(True)


if __name__ == "__main__":
    AppHelper.runEventLoop()



"""
Instances of WSTConnectionWindowController are the controlling object
for the document windows for the Web Services Tool application.

Implements a standard toolbar.
"""
import AppKit
import objc
from RPCMethod import RPCMethod
from twisted.internet import defer
from twisted.web.xmlrpc import Proxy

# from twisted.python import log
# import sys
# log.startLogging(sys.stdout)

# cheap dirty way to turn those messages off
# from twisted.python import log
# log.logerr = open('/dev/null','w')

# Identifier for 'reload contents' toolbar item.
kWSTReloadContentsToolbarItemIdentifier = "WST: Reload Contents Toolbar Identifier"

# Identifier for 'preferences' toolbar item.
kWSTPreferencesToolbarItemIdentifier = "WST: Preferences Toolbar Identifier"

# Identifier for URL text field toolbar item.
kWSTUrlTextFieldToolbarItemIdentifier = "WST: URL Textfield Toolbar Identifier"


def addToolbarItem(
    aController,
    anIdentifier,
    aLabel,
    aPaletteLabel,
    aToolTip,
    aTarget,
    anAction,
    anItemContent,
    aMenu,
):
    """
    Adds an freshly created item to the toolbar defined by
    aController.  Makes a number of assumptions about the
    implementation of aController.  It should be refactored into a
    generically useful toolbar management untility.
    """
    toolbarItem = AppKit.NSToolbarItem.alloc().initWithItemIdentifier_(anIdentifier)

    toolbarItem.setLabel_(aLabel)
    toolbarItem.setPaletteLabel_(aPaletteLabel)
    toolbarItem.setToolTip_(aToolTip)
    toolbarItem.setTarget_(aTarget)
    if anAction:
        toolbarItem.setAction_(anAction)

    if isinstance(anItemContent, AppKit.NSImage):
        toolbarItem.setImage_(anItemContent)
    else:
        toolbarItem.setView_(anItemContent)
        bounds = anItemContent.bounds()
        minSize = (100, bounds[1][1])
        maxSize = (1000, bounds[1][1])
        toolbarItem.setMinSize_(minSize)
        toolbarItem.setMaxSize_(maxSize)

    if aMenu:
        menuItem = AppKit.NSMenuItem.alloc().init()
        menuItem.setSubmenu_(aMenu)
        menuItem.setTitle_(aMenu.title())
        toolbarItem.setMenuFormRepresentation_(menuItem)

    aController.k_toolbarItems[anIdentifier] = toolbarItem


class WSTConnectionWindowController(AppKit.NSWindowController):
    methodDescriptionTextView = objc.IBOutlet()
    methodsTable = objc.IBOutlet()
    progressIndicator = objc.IBOutlet()
    statusTextField = objc.IBOutlet()
    urlTextField = objc.IBOutlet()

    @classmethod
    def connectionWindowController(self):
        """
        Create and return a default connection window instance.
        """
        return WSTConnectionWindowController.alloc().init()

    def init(self):
        """
        Designated initializer.

        Returns self (as per ObjC designated initializer definition,
        unlike Python's __init__() method).
        """
        self = self.initWithWindowNibName_("WSTConnection")

        self.k_toolbarItems = {}
        self.k_toolbarDefaultItemIdentifiers = []
        self.k_toolbarAllowedItemIdentifiers = []

        self.k_methods = {}
        self.k_methodArray = []
        return self

    def awakeFromNib(self):
        """
        Invoked when the NIB file is loaded.  Initializes the various
        UI widgets.
        """
        self.retain()  # balanced by autorelease() in windowWillClose_

        self.statusTextField.setStringValue_("No host specified.")
        self.progressIndicator.setStyle_(AppKit.NSProgressIndicatorSpinningStyle)
        self.progressIndicator.setDisplayedWhenStopped_(False)

        self.createToolbar()

    def windowWillClose_(self, aNotification):
        """
        Clean up when the document window is closed.
        """
        self.autorelease()

    def createToolbar(self):
        """
        Creates and configures the toolbar to be used by the window.
        """
        toolbar = AppKit.NSToolbar.alloc().initWithIdentifier_("WST Connection Window")
        toolbar.setDelegate_(self)
        toolbar.setAllowsUserCustomization_(True)
        toolbar.setAutosavesConfiguration_(True)

        self.createToolbarItems()

        self.window().setToolbar_(toolbar)

        lastURL = AppKit.NSUserDefaults.standardUserDefaults().stringForKey_("LastURL")
        if lastURL and len(lastURL):
            self.urlTextField.setStringValue_(lastURL)

    def createToolbarItems(self):
        """
        Creates all of the toolbar items that can be made available in
        the toolbar.  The actual set of available toolbar items is
        determined by other mechanisms (user defaults, for example).
        """
        addToolbarItem(
            self,
            kWSTReloadContentsToolbarItemIdentifier,
            "Reload",
            "Reload",
            "Reload Contents",
            None,
            "reloadVisibleData:",
            AppKit.NSImage.imageNamed_("Reload"),
            None,
        )
        addToolbarItem(
            self,
            kWSTPreferencesToolbarItemIdentifier,
            "Preferences",
            "Preferences",
            "Show Preferences",
            None,
            "orderFrontPreferences:",
            AppKit.NSImage.imageNamed_("Preferences"),
            None,
        )
        addToolbarItem(
            self,
            kWSTUrlTextFieldToolbarItemIdentifier,
            "URL",
            "URL",
            "Server URL",
            None,
            None,
            self.urlTextField,
            None,
        )

        self.k_toolbarDefaultItemIdentifiers = [
            kWSTReloadContentsToolbarItemIdentifier,
            kWSTUrlTextFieldToolbarItemIdentifier,
            AppKit.NSToolbarSeparatorItemIdentifier,
            AppKit.NSToolbarCustomizeToolbarItemIdentifier,
        ]

        self.k_toolbarAllowedItemIdentifiers = [
            kWSTReloadContentsToolbarItemIdentifier,
            kWSTUrlTextFieldToolbarItemIdentifier,
            AppKit.NSToolbarSeparatorItemIdentifier,
            AppKit.NSToolbarSpaceItemIdentifier,
            AppKit.NSToolbarFlexibleSpaceItemIdentifier,
            AppKit.NSToolbarPrintItemIdentifier,
            kWSTPreferencesToolbarItemIdentifier,
            AppKit.NSToolbarCustomizeToolbarItemIdentifier,
        ]

    def toolbarDefaultItemIdentifiers_(self, anIdentifier):
        """
        Return an array of toolbar item identifiers that identify the
        set, in order, of items that should be displayed on the
        default toolbar.
        """
        return self.k_toolbarDefaultItemIdentifiers

    def toolbarAllowedItemIdentifiers_(self, anIdentifier):
        """
        Return an array of toolbar items that may be used in the toolbar.
        """
        return self.k_toolbarAllowedItemIdentifiers

    def toolbar_itemForItemIdentifier_willBeInsertedIntoToolbar_(
        self, toolbar, itemIdentifier, flag
    ):
        """
        Delegate method fired when the toolbar is about to insert an
        item into the toolbar.  Item is identified by itemIdentifier.

        Effectively makes a copy of the cached reference instance of
        the toolbar item identified by itemIdentifier.
        """
        newItem = AppKit.NSToolbarItem.alloc().initWithItemIdentifier_(itemIdentifier)
        item = self.k_toolbarItems[itemIdentifier]

        newItem.setLabel_(item.label())
        newItem.setPaletteLabel_(item.paletteLabel())
        if item.view():
            newItem.setView_(item.view())
        else:
            newItem.setImage_(item.image())

        newItem.setToolTip_(item.toolTip())
        newItem.setTarget_(item.target())
        newItem.setAction_(item.action())
        newItem.setMenuFormRepresentation_(item.menuFormRepresentation())

        if newItem.view():
            newItem.setMinSize_(item.minSize())
            newItem.setMaxSize_(item.maxSize())

        return newItem

    def setStatusTextFieldMessage_(self, aMessage):
        """
        Sets the contents of the statusTextField to aMessage and
        forces the fileld's contents to be redisplayed.
        """
        if not aMessage:
            aMessage = "Displaying information about %d methods." % (
                len(self.k_methods),
            )
        self.statusTextField.setStringValue_(aMessage)

    setStatusTextFieldMessage_ = objc.accessor(setStatusTextFieldMessage_)

    def startWorking(self):
        """Signal the UI there's work going on."""
        self.progressIndicator.startAnimation_(self)

    def stopWorking(self):
        """Signal the UI that the work is done."""
        self.progressIndicator.stopAnimation_(self)

    @objc.IBAction
    def reloadVisibleData_(self, sender):
        """
        Reloads the list of methods and their signatures from the
        XML-RPC server specified in the urlTextField.  Displays
        appropriate error messages, if necessary.
        """
        url = self.urlTextField.stringValue()
        self.k_methods = {}

        if not url:
            self.window().setTitle_("Untitled.")
            self.setStatusTextFieldMessage_("No URL specified.")
            return

        self.window().setTitle_(url)
        AppKit.NSUserDefaults.standardUserDefaults().setObject_forKey_(url, "LastURL")

        self.setStatusTextFieldMessage_("Retrieving method list...")
        self.getMethods(url)

    @objc.python_method
    def getMethods(self, url):
        _server = self.k_server = Proxy(url.encode("utf8"))
        self.startWorking()
        return (
            _server.callRemote("listMethods")
            .addCallback(
                # call self.receivedMethods(result, _server, "") on success
                self.receivedMethods,
                _server,
                "",
            )
            .addErrback(
                # on error, call this lambda
                lambda e: _server.callRemote("system.listMethods").addCallback(
                    # call self.receievedMethods(result, _server, "system.")
                    self.receivedMethods,
                    _server,
                    "system.",
                )
            )
            .addErrback(
                # log the failure instance, with a method
                self.receivedMethodsFailure,
                "listMethods()",
            )
            .addBoth(
                # stop working nomatter what trap all errors (returns None)
                lambda n: self.stopWorking()
            )
        )

    @objc.python_method
    def receivedMethodsFailure(self, why, method):
        self.k_server = None
        self.k_methodPrefix = None
        self.setStatusTextFieldMessage_(
            ("Server failed to respond to %s.  " "See below for more information.")
            % (method,)
        )
        # log.err(why)
        self.methodDescriptionTextView.setString_(why.getTraceback())

    @objc.python_method
    def receivedMethods(self, _methods, _server, _methodPrefix):
        self.k_server = _server
        self.k_methods = {}
        self.k_methodPrefix = _methodPrefix
        for aMethod in _methods:
            self.k_methods[aMethod] = RPCMethod.alloc().initWithDocument_name_(
                self, aMethod
            )
        self.setMethodArray_(self.k_methods.values())
        self.k_methodPrefix = _methodPrefix

        self.setStatusTextFieldMessage_(
            "Retrieving information about %d methods." % (len(self.k_methods),)
        )

        # we could make all the requests at once :)
        # but the server might not like that so we will chain them
        d = defer.succeed(None)
        for index, aMethod in enumerate(self.k_methodArray):
            d.addCallback(self.fetchMethodSignature, index, aMethod).addCallbacks(
                callback=self.processSignatureForMethod,
                callbackArgs=(index, aMethod),
                errback=self.couldntProcessSignatureForMethod,
                errbackArgs=(index, aMethod),
            )
        return d.addCallback(lambda ig: self.setStatusTextFieldMessage_(None))

    @objc.python_method
    def fetchMethodSignature(self, ignore, index, aMethod):
        self.setStatusTextFieldMessage_(
            "Retrieving signature for method %s (%d of %d)."
            % (aMethod.methodName(), index, len(self.k_methods))
        )
        return self.k_server.callRemote(
            (self.k_methodPrefix + "methodSignature").encode("utf-8"),
            aMethod.methodName().encode("utf-8"),
        )

    @objc.python_method
    def processSignatureForMethod(self, methodSignature, index, aMethod):
        signatures = None
        if not len(methodSignature):
            return
        for aSignature in methodSignature:
            if isinstance(aSignature, list) and len(aSignature) > 0:
                signature = "{} {}({})".format(
                    aSignature[0],
                    aMethod.methodName(),
                    ", ".join(aSignature[1:]),
                )
            else:
                signature = aSignature
        if signatures:
            signatures = signatures + ", " + signature
        else:
            signatures = signature

        aMethod.setMethodSignature_(signatures)
        self.replaceObjectInMethodArrayAtIndex_withObject_(index, aMethod)

    @objc.python_method
    def couldntProcessSignatureForMethod(self, why, index, aMethod):
        # log.err(why)
        aMethod.setMethodSignature_(
            f"<error> {aMethod.methodName()} {why.getBriefTraceback()}"
        )
        self.replaceObjectInMethodArrayAtIndex_withObject_(index, aMethod)

    def fetchMethodDescription_(self, aMethod):
        def cacheDesc(v):
            aMethod.setMethodDescription_(v or "No description available.")

        self.setStatusTextFieldMessage_(
            f"Retrieving documentation for method {aMethod.methodName()}..."
        )
        self.startWorking()
        self.k_server.callRemote(
            (self.k_methodPrefix + "methodHelp").encode("utf-8"),
            aMethod.methodName().encode("utf-8"),
        ).addCallback(cacheDesc)

    def methodArray(self):
        return self.k_methodArray

    @objc.accessor
    def countOfMethodArray(self):
        if self.k_methodArray is None:
            return 0
        return self.k_methodArray

    @objc.accessor
    def objectInMethodArrayAtIndex_(self, anIndex):
        return self.k_methodArray[anIndex]

    @objc.accessor
    def insertObject_inMethodArrayAtIndex_(self, anObject, anIndex):
        self.k_methodArray.insert(anIndex, anObject)

    @objc.accessor
    def removeObjectFromMethodArrayAtIndex_(self, anIndex):
        del self.k_methodArray[anIndex]

    @objc.accessor
    def replaceObjectInMethodArrayAtIndex_withObject_(self, anIndex, anObject):
        self.k_methodArray[anIndex] = anObject

    @objc.accessor
    def setMethodArray_(self, anArray):
        self.k_methodArray = anArray



#!/usr/bin/python

import objc
from Foundation import NSKeyValueChangeNewKey, NSKeyValueObservingOptionNew, NSObject


class MyClass(NSObject):
    base = objc.ivar("base", objc._C_INT)
    power = objc.ivar("power", objc._C_INT)
    result = objc.ivar("result", objc._C_INT)

    def result(self):
        return self.base ** self.power


MyClass.setKeys_triggerChangeNotificationsForDependentKey_(["base", "power"], "result")


class Observer(NSObject):
    def observeValueForKeyPath_ofObject_change_context_(
        self, path, an_object, changeDescription, context
    ):
        print(
            'path "%s" was changed to "%s".'
            % (path, changeDescription[NSKeyValueChangeNewKey])
        )


myInstance = MyClass.new()
observer = Observer.new()

myInstance.addObserver_forKeyPath_options_context_(
    observer, "result", NSKeyValueObservingOptionNew, 0
)
myInstance.addObserver_forKeyPath_options_context_(
    observer, "base", NSKeyValueObservingOptionNew, 0
)
myInstance.addObserver_forKeyPath_options_context_(
    observer, "power", NSKeyValueObservingOptionNew, 0
)

myInstance.setValue_forKey_(2, "base")
myInstance.power = 4

print(
    "%d ** %d == %d"
    % (myInstance.base, myInstance.valueForKey_("power"), myInstance.result())
)



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase


class TestNSComparisonPredicate(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSCaseInsensitivePredicateOption, 1)
        self.assertEqual(Foundation.NSDiacriticInsensitivePredicateOption, 2)
        self.assertEqual(Foundation.NSNormalizedPredicateOption, 4)

        self.assertEqual(Foundation.NSDirectPredicateModifier, 0)
        self.assertEqual(Foundation.NSAllPredicateModifier, 1)
        self.assertEqual(Foundation.NSAnyPredicateModifier, 2)

        self.assertEqual(Foundation.NSLessThanPredicateOperatorType, 0)
        self.assertEqual(Foundation.NSLessThanOrEqualToPredicateOperatorType, 1)
        self.assertEqual(Foundation.NSGreaterThanPredicateOperatorType, 2)
        self.assertEqual(Foundation.NSGreaterThanOrEqualToPredicateOperatorType, 3)
        self.assertEqual(Foundation.NSEqualToPredicateOperatorType, 4)
        self.assertEqual(Foundation.NSNotEqualToPredicateOperatorType, 5)
        self.assertEqual(Foundation.NSMatchesPredicateOperatorType, 6)
        self.assertEqual(Foundation.NSLikePredicateOperatorType, 7)
        self.assertEqual(Foundation.NSBeginsWithPredicateOperatorType, 8)
        self.assertEqual(Foundation.NSEndsWithPredicateOperatorType, 9)
        self.assertEqual(Foundation.NSInPredicateOperatorType, 10)
        self.assertEqual(Foundation.NSCustomSelectorPredicateOperatorType, 11)
        self.assertEqual(Foundation.NSContainsPredicateOperatorType, 99)
        self.assertEqual(Foundation.NSBetweenPredicateOperatorType, 100)

    def testMethods(self):
        self.assertArgIsSEL(
            Foundation.NSComparisonPredicate.predicateWithLeftExpression_rightExpression_customSelector_,  # noqa: B950
            2,
            objc._C_NSBOOL + b"@:@",
        )
        self.assertArgIsSEL(
            Foundation.NSComparisonPredicate.initWithLeftExpression_rightExpression_customSelector_,  # noqa: B950
            2,
            objc._C_NSBOOL + b"@:@",
        )
        self.assertResultIsSEL(
            Foundation.NSComparisonPredicate.customSelector, objc._C_NSBOOL + b"@:@"
        )



import Foundation
from PyObjCTest.testhelper import PyObjC_TestClass3
from PyObjCTools.TestSupport import TestCase


class TestNSHost(TestCase):
    def testCreation(self):
        #
        # This gives an exception on GNUstep:
        #   Foundation.NSRecursiveLockException - unlock: failed to unlock mutex
        #
        # testIndirectCreation performs the same operation from Objective-C
        # and doesn't fail. I don't see any significant differences in the
        # implementation of -hostWithAddress: and -hostWithName: yet the latter
        # does not have the problem we're seeing here.
        #
        o = Foundation.NSHost.hostWithAddress_("127.0.0.1")
        self.assertEqual(o.addresses(), ("127.0.0.1",))
        self.assertEqual(o.address(), "127.0.0.1")

    def testCreation2(self):
        o = Foundation.NSHost.hostWithName_("localhost")
        lst = sorted(o.addresses())
        self.assertIn("127.0.0.1", lst)
        self.assertEqual(o.address(), o.addresses()[0])

    def testIndirectCreation(self):
        o = PyObjC_TestClass3.createAHostWithAddress_("127.0.0.1")
        self.assertEqual(o.address(), "127.0.0.1")

    def testMethods(self):
        self.assertArgIsBOOL(Foundation.NSHost.setHostCacheEnabled_, 0)
        self.assertResultIsBOOL(Foundation.NSHost.isHostCacheEnabled)
        self.assertResultIsBOOL(Foundation.NSHost.isEqualToHost_)



#
#  RadiansToDegreesTransformer.py
#  GraphicsBindings
#
#  Converted by u.fiedler on feb 2005
#  with great help from Bob Ippolito - Thank you Bob!
#
#  The original version was written in Objective-C by Malcolm Crawford
#  http://homepage.mac.com/mmalc/CocoaExamples/controllers.html

from Foundation import NSNumber, NSValueTransformer


class RadiansToDegreesTransformer(NSValueTransformer):
    @classmethod
    def transformedValueClass(cls):
        return NSNumber

    @classmethod
    def allowsReverseTransformation(cls):
        return True

    def transformedValue_(self, radians):
        return radians / (3.141_592_7 / 180.0)

    def reverseTransformedValue_(self, degrees):
        if isinstance(degrees, float):
            # when using jostickview we get a value of type float()
            return degrees * (3.141_592_7 / 180.0)
        else:
            # we get a decimalNumber when entering a value in the textfield
            return degrees.doubleValue() * (3.141_592_7 / 180.0)



import AppKit  # noqa: F401
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestNSAlignmentFeedbackFilter(TestCase):
    @min_os_level("10.11")
    def testProtocols(self):
        objc.protocolNamed("NSAlignmentFeedbackToken")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSButtonCell(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSMomentaryLightButton, 0)
        self.assertEqual(AppKit.NSPushOnPushOffButton, 1)
        self.assertEqual(AppKit.NSToggleButton, 2)
        self.assertEqual(AppKit.NSSwitchButton, 3)
        self.assertEqual(AppKit.NSRadioButton, 4)
        self.assertEqual(AppKit.NSMomentaryChangeButton, 5)
        self.assertEqual(AppKit.NSOnOffButton, 6)
        self.assertEqual(AppKit.NSMomentaryPushInButton, 7)
        self.assertEqual(AppKit.NSAcceleratorButton, 8)
        self.assertEqual(AppKit.NSMultiLevelAcceleratorButton, 9)

        self.assertEqual(AppKit.NSMomentaryPushButton, 0)
        self.assertEqual(AppKit.NSMomentaryLight, 7)

        self.assertEqual(AppKit.NSRoundedBezelStyle, 1)
        self.assertEqual(AppKit.NSRegularSquareBezelStyle, 2)
        self.assertEqual(AppKit.NSThickSquareBezelStyle, 3)
        self.assertEqual(AppKit.NSThickerSquareBezelStyle, 4)
        self.assertEqual(AppKit.NSDisclosureBezelStyle, 5)
        self.assertEqual(AppKit.NSShadowlessSquareBezelStyle, 6)
        self.assertEqual(AppKit.NSCircularBezelStyle, 7)
        self.assertEqual(AppKit.NSTexturedSquareBezelStyle, 8)
        self.assertEqual(AppKit.NSHelpButtonBezelStyle, 9)
        self.assertEqual(AppKit.NSSmallSquareBezelStyle, 10)
        self.assertEqual(AppKit.NSTexturedRoundedBezelStyle, 11)
        self.assertEqual(AppKit.NSRoundRectBezelStyle, 12)
        self.assertEqual(AppKit.NSRecessedBezelStyle, 13)
        self.assertEqual(AppKit.NSRoundedDisclosureBezelStyle, 14)
        self.assertEqual(AppKit.NSSmallIconButtonBezelStyle, 2)

        self.assertEqual(AppKit.NSGradientNone, 0)
        self.assertEqual(AppKit.NSGradientConcaveWeak, 1)
        self.assertEqual(AppKit.NSGradientConcaveStrong, 2)
        self.assertEqual(AppKit.NSGradientConvexWeak, 3)
        self.assertEqual(AppKit.NSGradientConvexStrong, 4)

        self.assertEqual(AppKit.NSButtonTypeMomentaryLight, 0)
        self.assertEqual(AppKit.NSButtonTypePushOnPushOff, 1)
        self.assertEqual(AppKit.NSButtonTypeToggle, 2)
        self.assertEqual(AppKit.NSButtonTypeSwitch, 3)
        self.assertEqual(AppKit.NSButtonTypeRadio, 4)
        self.assertEqual(AppKit.NSButtonTypeMomentaryChange, 5)
        self.assertEqual(AppKit.NSButtonTypeOnOff, 6)
        self.assertEqual(AppKit.NSButtonTypeMomentaryPushIn, 7)
        self.assertEqual(AppKit.NSButtonTypeAccelerator, 8)
        self.assertEqual(AppKit.NSButtonTypeMultiLevelAccelerator, 9)

        self.assertEqual(AppKit.NSBezelStyleRounded, 1)
        self.assertEqual(AppKit.NSBezelStyleRegularSquare, 2)
        self.assertEqual(AppKit.NSBezelStyleDisclosure, 5)
        self.assertEqual(AppKit.NSBezelStyleShadowlessSquare, 6)
        self.assertEqual(AppKit.NSBezelStyleCircular, 7)
        self.assertEqual(AppKit.NSBezelStyleTexturedSquare, 8)
        self.assertEqual(AppKit.NSBezelStyleHelpButton, 9)
        self.assertEqual(AppKit.NSBezelStyleSmallSquare, 10)
        self.assertEqual(AppKit.NSBezelStyleTexturedRounded, 11)
        self.assertEqual(AppKit.NSBezelStyleRoundRect, 12)
        self.assertEqual(AppKit.NSBezelStyleRecessed, 13)
        self.assertEqual(AppKit.NSBezelStyleRoundedDisclosure, 14)
        self.assertEqual(AppKit.NSBezelStyleInline, 15)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSInlineBezelStyle, 15)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSButtonCell.isOpaque)
        self.assertResultIsBOOL(AppKit.NSButtonCell.isTransparent)
        self.assertArgIsBOOL(AppKit.NSButtonCell.setTransparent_, 0)

        self.assertArgIsOut(AppKit.NSButtonCell.getPeriodicDelay_interval_, 0)
        self.assertArgIsOut(AppKit.NSButtonCell.getPeriodicDelay_interval_, 1)

        self.assertResultIsBOOL(AppKit.NSButtonCell.imageDimsWhenDisabled)
        self.assertArgIsBOOL(AppKit.NSButtonCell.setImageDimsWhenDisabled_, 0)
        self.assertResultIsBOOL(AppKit.NSButtonCell.showsBorderOnlyWhileMouseInside)
        self.assertArgIsBOOL(AppKit.NSButtonCell.setShowsBorderOnlyWhileMouseInside_, 0)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTitlebarAccessoryViewController(TestCase):
    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(AppKit.NSTitlebarAccessoryViewController.isHidden)
        self.assertArgIsBOOL(AppKit.NSTitlebarAccessoryViewController.setHidden_, 0)



import Foundation
import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSLocale(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSCurrentLocaleDidChangeNotification, str)
        self.assertIsInstance(Foundation.NSLocaleIdentifier, str)
        self.assertIsInstance(Foundation.NSLocaleLanguageCode, str)
        self.assertIsInstance(Foundation.NSLocaleCountryCode, str)
        self.assertIsInstance(Foundation.NSLocaleScriptCode, str)
        self.assertIsInstance(Foundation.NSLocaleVariantCode, str)
        self.assertIsInstance(Foundation.NSLocaleExemplarCharacterSet, str)
        self.assertIsInstance(Foundation.NSLocaleCalendar, str)
        self.assertIsInstance(Foundation.NSLocaleCollationIdentifier, str)
        self.assertIsInstance(Foundation.NSLocaleUsesMetricSystem, str)
        self.assertIsInstance(Foundation.NSLocaleMeasurementSystem, str)
        self.assertIsInstance(Foundation.NSLocaleDecimalSeparator, str)
        self.assertIsInstance(Foundation.NSLocaleGroupingSeparator, str)
        self.assertIsInstance(Foundation.NSLocaleCurrencySymbol, str)
        self.assertIsInstance(Foundation.NSLocaleCurrencyCode, str)
        self.assertIsInstance(Foundation.NSGregorianCalendar, str)
        self.assertIsInstance(Foundation.NSBuddhistCalendar, str)
        self.assertIsInstance(Foundation.NSChineseCalendar, str)
        self.assertIsInstance(Foundation.NSHebrewCalendar, str)
        self.assertIsInstance(Foundation.NSIslamicCalendar, str)
        self.assertIsInstance(Foundation.NSIslamicCivilCalendar, str)
        self.assertIsInstance(Foundation.NSJapaneseCalendar, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(Foundation.NSLocaleCollatorIdentifier, str)
        self.assertIsInstance(Foundation.NSLocaleQuotationBeginDelimiterKey, str)
        self.assertIsInstance(Foundation.NSLocaleQuotationEndDelimiterKey, str)
        self.assertIsInstance(
            Foundation.NSLocaleAlternateQuotationBeginDelimiterKey, str
        )
        self.assertIsInstance(Foundation.NSLocaleAlternateQuotationEndDelimiterKey, str)
        self.assertIsInstance(Foundation.NSRepublicOfChinaCalendar, str)
        self.assertIsInstance(Foundation.NSPersianCalendar, str)
        self.assertIsInstance(Foundation.NSIndianCalendar, str)
        self.assertIsInstance(Foundation.NSISO8601Calendar, str)

        self.assertEqual(
            Foundation.NSLocaleLanguageDirectionUnknown,
            CoreFoundation.kCFLocaleLanguageDirectionUnknown,
        )
        self.assertEqual(
            Foundation.NSLocaleLanguageDirectionLeftToRight,
            CoreFoundation.kCFLocaleLanguageDirectionLeftToRight,
        )
        self.assertEqual(
            Foundation.NSLocaleLanguageDirectionRightToLeft,
            CoreFoundation.kCFLocaleLanguageDirectionRightToLeft,
        )
        self.assertEqual(
            Foundation.NSLocaleLanguageDirectionTopToBottom,
            CoreFoundation.kCFLocaleLanguageDirectionTopToBottom,
        )
        self.assertEqual(
            Foundation.NSLocaleLanguageDirectionBottomToTop,
            CoreFoundation.kCFLocaleLanguageDirectionBottomToTop,
        )



#!/usr/bin/env python
# This is a doctest
"""
==============================
Using Cocoa collection classes
==============================

Cocoa contains a number of collection classes, including dictionaries and
arrays (like python lists) in mutable and immutable variations. We'll
demonstrate their usage using the ``NSMutableDictonary`` class.

We'll start by importing everything we need::

    >>> from Foundation import NSMutableDictionary

Then create an empty dictionary::

    >>> d = NSMutableDictionary.dictionary()
    >>> d
    {
    }
    >>> isinstance(d, dict)
    False
    >>> isinstance(d, NSMutableDictionary)
    True

You can add a new value using the Objective-C API::

    >>> d.setObject_forKey_(42, 'key2')
    >>> d
    {
        key2 = 42;
    }

But can also use the familiar python interface:

    >>> d['key1'] = 'hello'
    >>> d
    {
        key1 = hello;
        key2 = 42;
    }

The same is true for fetching elements::

    >>> d['key2']
    42
    >>> d.objectForKey_('key1')
    'hello'
"""
import doctest

import __main__

doctest.testmod(__main__, verbose=1)



import Cocoa
import objc


def serviceSelector(fn):
    # this is the signature of service selectors
    return objc.selector(fn, signature=b"v@:@@o^@")


def ERROR(s):
    # NSLog(u"ERROR: %s", s)
    return s


class ServiceTest(Cocoa.NSObject):
    @serviceSelector
    def doOpenFileService_userData_error_(self, pboard, data, error):
        try:
            types = pboard.types()
            pboardString = None
            if Cocoa.NSStringPboardType in types:
                pboardString = pboard.stringForType_(Cocoa.NSStringPboardType)
            if pboardString is None:
                return ERROR(
                    Cocoa.NSLocalizedString(
                        "Error: Pasteboard doesn't contain a string.",
                        "Pasteboard couldn't give string.",
                    )
                )

            if not Cocoa.NSWorkspace.sharedWorkspace().openFile_(pboardString):
                return ERROR(
                    Cocoa.NSLocalizedString(
                        "Error: Couldn't open file %s.",
                        "Couldn't perform service operation for file %s.",
                    )
                    % pboardString
                )

            return ERROR(None)
        except:  # noqa: E722, B001
            import traceback

            traceback.print_exc()
            return ERROR("Exception, see traceback")

    @serviceSelector
    def doCapitalizeService_userData_error_(self, pboard, data, err):
        # NSLog(u"doCapitalizeService_userData_error_(%s, %s)", pboard, data)
        try:
            types = pboard.types()
            pboardString = None
            if Cocoa.NSStringPboardType in types:
                pboardString = pboard.stringForType_(Cocoa.NSStringPboardType)
            if pboardString is None:
                return ERROR(
                    Cocoa.NSLocalizedString(
                        "Error: Pasteboard doesn't contain a string.",
                        "Pasteboard couldn't give string.",
                    )
                )

            newString = Cocoa.NSString.capitalizedString(pboardString)

            if not newString:
                return ERROR(
                    Cocoa.NSLocalizedString(
                        "Error: Couldn't capitalize string %s.",
                        "Couldn't perform service operation for string %s.",
                    )
                    % pboardString
                )

            types = [Cocoa.NSStringPboardType]
            pboard.declareTypes_owner_([Cocoa.NSStringPboardType], None)
            pboard.setString_forType_(newString, Cocoa.NSStringPboardType)
            return ERROR(None)
        except:  # noqa: E722, B001
            import traceback

            traceback.print_exc()
            return ERROR("Exception, see traceback")



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSColorWell(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSColorWell.isActive)
        self.assertArgIsBOOL(AppKit.NSColorWell.activate_, 0)
        self.assertResultIsBOOL(AppKit.NSColorWell.isBordered)
        self.assertArgIsBOOL(AppKit.NSColorWell.setBordered_, 0)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase


class TestNSInputManagerHelper(AppKit.NSObject):
    def doCommandBySelector_(self, s):
        pass

    def setMarkedText_selectedRange_(self, s, r):
        pass

    def hasMarkedText(self):
        return 1

    def conversationIdentifier(self):
        return 1

    def markedRange(self):
        return 1

    def selectedRange(self):
        return 1

    def firstRectForCharacterRange_(self, r):
        return 1

    def characterIndexForPoint_(self, r):
        return 1


class TestNSInputManager(TestCase):
    def testProtocols(self):
        objc.protocolNamed("NSTextInput")
        self.assertArgIsSEL(TestNSInputManagerHelper.doCommandBySelector_, 0, b"v@:@")
        self.assertArgHasType(
            TestNSInputManagerHelper.setMarkedText_selectedRange_,
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultIsBOOL(TestNSInputManagerHelper.hasMarkedText)
        self.assertResultHasType(
            TestNSInputManagerHelper.markedRange, AppKit.NSRange.__typestr__
        )
        self.assertResultHasType(
            TestNSInputManagerHelper.selectedRange, AppKit.NSRange.__typestr__
        )
        self.assertResultHasType(
            TestNSInputManagerHelper.firstRectForCharacterRange_,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSInputManagerHelper.firstRectForCharacterRange_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertResultHasType(
            TestNSInputManagerHelper.characterIndexForPoint_, objc._C_NSUInteger
        )
        self.assertArgHasType(
            TestNSInputManagerHelper.characterIndexForPoint_,
            0,
            AppKit.NSPoint.__typestr__,
        )

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSInputManager.wantsToInterpretAllKeystrokes)
        self.assertResultIsBOOL(AppKit.NSInputManager.wantsToHandleMouseEvents)
        self.assertResultIsBOOL(AppKit.NSInputManager.handleMouseEvent_)
        self.assertResultIsBOOL(
            AppKit.NSInputManager.wantsToDelayTextChangeNotifications
        )



"""
Python mapping for the Foundation framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""

import objc as _objc

__bundle__ = _objc.initFrameworkWrapper(
    "CGL",
    frameworkIdentifier="com.apple.opengl",
    frameworkPath=_objc.pathForFramework("/System/Library/Frameworks/OpenGL.framework"),
    globals=globals(),
)



import CoreFoundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestLocale(TestCase):
    def testTypes(self):
        try:
            cls = objc.lookUpClass("__NSCFLocale")
            self.assertIs(CoreFoundation.CFLocaleRef, cls)
        except objc.error:
            try:
                cls = objc.lookUpClass("NSCFLocale")
                self.assertIs(CoreFoundation.CFLocaleRef, cls)
            except objc.error:
                self.assertIsCFType(CoreFoundation.CFLocaleRef)

    def testGetTypeID(self):
        self.assertIsInstance(CoreFoundation.CFLocaleGetTypeID(), int)

    def testInspection(self):
        locale = CoreFoundation.CFLocaleGetSystem()
        self.assertIsInstance(locale, CoreFoundation.CFLocaleRef)
        locale = CoreFoundation.CFLocaleCopyCurrent()
        self.assertIsInstance(locale, CoreFoundation.CFLocaleRef)
        idents = CoreFoundation.CFLocaleCopyAvailableLocaleIdentifiers()
        self.assertIsInstance(idents, CoreFoundation.CFArrayRef)
        codes = CoreFoundation.CFLocaleCopyISOLanguageCodes()
        self.assertIsInstance(codes, CoreFoundation.CFArrayRef)
        codes = CoreFoundation.CFLocaleCopyISOCountryCodes()
        self.assertIsInstance(codes, CoreFoundation.CFArrayRef)
        codes = CoreFoundation.CFLocaleCopyISOCurrencyCodes()
        self.assertIsInstance(codes, CoreFoundation.CFArrayRef)
        val = CoreFoundation.CFLocaleCreateCanonicalLanguageIdentifierFromString(
            None, "de_DE"
        )
        self.assertIsInstance(val, str)
        self.assertEqual(val, "de-DE")
        val = CoreFoundation.CFLocaleCreateCanonicalLocaleIdentifierFromString(
            None, "de_DE"
        )
        self.assertIsInstance(val, str)
        self.assertEqual(val, "de_DE")

        val = CoreFoundation.CFLocaleCreateCanonicalLocaleIdentifierFromScriptManagerCodes(
            None, 55, 75
        )
        self.assertIsInstance(val, str)
        dct = CoreFoundation.CFLocaleCreateComponentsFromLocaleIdentifier(None, "nl_NL")
        self.assertEqual(dct[CoreFoundation.kCFLocaleCountryCodeKey], "NL")
        self.assertEqual(dct[CoreFoundation.kCFLocaleLanguageCodeKey], "nl")
        val = CoreFoundation.CFLocaleCreateLocaleIdentifierFromComponents(None, dct)
        self.assertIsInstance(val, str)
        self.assertEqual(val, "nl_NL")

        locale = CoreFoundation.CFLocaleCreate(None, "nl_NL")
        self.assertIsInstance(locale, CoreFoundation.CFLocaleRef)
        locale = CoreFoundation.CFLocaleCreateCopy(None, locale)
        self.assertIsInstance(locale, CoreFoundation.CFLocaleRef)
        ident = CoreFoundation.CFLocaleGetIdentifier(locale)
        self.assertEqual(ident, "nl_NL")
        v = CoreFoundation.CFLocaleGetValue(
            locale, CoreFoundation.kCFLocaleDecimalSeparator
        )
        self.assertEqual(v, ",")
        v = CoreFoundation.CFLocaleCopyDisplayNameForPropertyValue(
            locale, CoreFoundation.kCFLocaleIdentifier, "nl_NL"
        )
        if v is not None:
            self.assertIsInstance(v, str)
        self.assertEqual(v, "Nederlands (Nederland)")

    def testConstants(self):
        self.assertIsInstance(CoreFoundation.kCFLocaleIdentifier, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleLanguageCode, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleCountryCode, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleScriptCode, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleVariantCode, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleExemplarCharacterSet, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleCalendarIdentifier, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleCollationIdentifier, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleUsesMetricSystem, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleMeasurementSystem, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleDecimalSeparator, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleGroupingSeparator, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleCurrencySymbol, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleCurrencyCode, str)
        self.assertIsInstance(CoreFoundation.kCFGregorianCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFBuddhistCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFChineseCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFHebrewCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFIslamicCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFIslamicCivilCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFJapaneseCalendar, str)

    @min_os_level("10.5")
    def testFunctions10_5(self):
        codes = CoreFoundation.CFLocaleCopyCommonISOCurrencyCodes()
        self.assertIsInstance(codes, CoreFoundation.CFArrayRef)
        codes = CoreFoundation.CFLocaleCopyPreferredLanguages()
        self.assertIsInstance(codes, CoreFoundation.CFArrayRef)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(
            CoreFoundation.kCFLocaleCurrentLocaleDidChangeNotification, str
        )

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(CoreFoundation.kCFLocaleLanguageDirectionUnknown, 0)
        self.assertEqual(CoreFoundation.kCFLocaleLanguageDirectionLeftToRight, 1)
        self.assertEqual(CoreFoundation.kCFLocaleLanguageDirectionRightToLeft, 2)
        self.assertEqual(CoreFoundation.kCFLocaleLanguageDirectionTopToBottom, 3)
        self.assertEqual(CoreFoundation.kCFLocaleLanguageDirectionBottomToTop, 4)

        self.assertIsInstance(CoreFoundation.kCFLocaleCollatorIdentifier, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleQuotationBeginDelimiterKey, str)
        self.assertIsInstance(CoreFoundation.kCFLocaleQuotationEndDelimiterKey, str)
        self.assertIsInstance(
            CoreFoundation.kCFLocaleAlternateQuotationBeginDelimiterKey, str
        )
        self.assertIsInstance(
            CoreFoundation.kCFLocaleAlternateQuotationEndDelimiterKey, str
        )
        self.assertIsInstance(CoreFoundation.kCFRepublicOfChinaCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFPersianCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFIndianCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFISO8601Calendar, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(CoreFoundation.kCFIslamicTabularCalendar, str)
        self.assertIsInstance(CoreFoundation.kCFIslamicUmmAlQuraCalendar, str)

    @min_os_level("10.6")
    def testFunctions10_6(self):
        v = CoreFoundation.CFLocaleGetWindowsLocaleCodeFromLocaleIdentifier("nl_NL")
        self.assertIsInstance(v, int)

        self.assertResultIsCFRetained(
            CoreFoundation.CFLocaleCreateLocaleIdentifierFromWindowsLocaleCode
        )
        v = CoreFoundation.CFLocaleCreateLocaleIdentifierFromWindowsLocaleCode(
            None, 1043
        )
        self.assertIsInstance(v, str)

        v = CoreFoundation.CFLocaleGetLanguageCharacterDirection("NL")
        self.assertEqual(v, CoreFoundation.kCFLocaleLanguageDirectionLeftToRight)

        v = CoreFoundation.CFLocaleGetLanguageLineDirection("NL")
        self.assertEqual(v, CoreFoundation.kCFLocaleLanguageDirectionTopToBottom)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSScriptStandardSuiteCommands(TestCase):
    def testCommands(self):
        self.assertEqual(Foundation.NSSaveOptionsYes, 0)
        self.assertEqual(Foundation.NSSaveOptionsNo, 1)
        self.assertEqual(Foundation.NSSaveOptionsAsk, 2)



import array
import sys

import Foundation
from PyObjCTest.testhelper import PyObjC_TestClass3
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc

rawBytes = b"a\x13b\x00cd\xFFef\xEFgh"
otherBytes = array.array("B")
otherBytes.frombytes(b"12345678901234567890" * 5)


class TestNSData(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSData.isEqualToData_)
        self.assertResultIsBOOL(Foundation.NSData.writeToFile_atomically_)
        self.assertArgIsBOOL(Foundation.NSData.writeToFile_atomically_, 1)
        self.assertResultIsBOOL(Foundation.NSData.writeToURL_atomically_)
        self.assertArgIsBOOL(Foundation.NSData.writeToURL_atomically_, 1)
        self.assertResultIsBOOL(Foundation.NSData.writeToFile_options_error_)
        self.assertArgIsOut(Foundation.NSData.writeToFile_options_error_, 2)
        self.assertResultIsBOOL(Foundation.NSData.writeToURL_options_error_)
        self.assertArgIsOut(Foundation.NSData.writeToURL_options_error_, 2)
        self.assertArgIsOut(Foundation.NSData.dataWithContentsOfFile_options_error_, 2)
        self.assertArgIsOut(Foundation.NSData.dataWithContentsOfURL_options_error_, 2)
        self.assertArgIsOut(Foundation.NSData.initWithContentsOfFile_options_error_, 2)
        self.assertArgIsOut(Foundation.NSData.initWithContentsOfURL_options_error_, 2)
        self.assertArgIsIn(Foundation.NSData.dataWithBytesNoCopy_length_, 0)
        self.assertArgSizeInArg(Foundation.NSData.dataWithBytes_length_, 0, 1)
        self.assertArgIsIn(Foundation.NSData.initWithBytesNoCopy_length_, 0)
        self.assertArgSizeInArg(Foundation.NSData.initWithBytes_length_, 0, 1)

    def testConstants(self):
        self.assertEqual(Foundation.NSMappedRead, 1)
        self.assertEqual(Foundation.NSUncachedRead, 2)

        self.assertEqual(Foundation.NSAtomicWrite, 1)

        self.assertEqual(Foundation.NSDataWritingFileProtectionNone, 0x10000000)
        self.assertEqual(Foundation.NSDataWritingFileProtectionComplete, 0x20000000)
        self.assertEqual(
            Foundation.NSDataWritingFileProtectionCompleteUnlessOpen, 0x30000000
        )
        self.assertEqual(
            Foundation.NSDataWritingFileProtectionCompleteUntilFirstUserAuthentication,
            0x40000000,
        )
        self.assertEqual(Foundation.NSDataWritingFileProtectionMask, 0xF0000000)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(Foundation.NSDataReadingMapped, 1 << 0)
        self.assertEqual(Foundation.NSDataReadingUncached, 1 << 1)
        self.assertEqual(Foundation.NSDataWritingAtomic, 1 << 0)
        self.assertEqual(Foundation.NSDataSearchBackwards, 1 << 0)
        self.assertEqual(Foundation.NSDataSearchAnchored, 1 << 1)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(Foundation.NSDataReadingMappedAlways, 1 << 3)

        self.assertEqual(Foundation.NSDataReadingMappedIfSafe, 1 << 0)
        self.assertEqual(Foundation.NSDataReadingUncached, 1 << 1)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(Foundation.NSDataWritingWithoutOverwriting, 1 << 1)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertEqual(Foundation.NSDataBase64Encoding64CharacterLineLength, 1 << 0)
        self.assertEqual(Foundation.NSDataBase64Encoding76CharacterLineLength, 1 << 1)
        self.assertEqual(
            Foundation.NSDataBase64EncodingEndLineWithCarriageReturn, 1 << 4
        )
        self.assertEqual(Foundation.NSDataBase64EncodingEndLineWithLineFeed, 1 << 5)

        self.assertEqual(Foundation.NSDataBase64DecodingIgnoreUnknownCharacters, 1 << 0)

    @min_os_level("10.15")
    def testConstants10_15(self):
        self.assertEqual(Foundation.NSDataCompressionAlgorithmLZFSE, 0)
        self.assertEqual(Foundation.NSDataCompressionAlgorithmLZ4, 1)
        self.assertEqual(Foundation.NSDataCompressionAlgorithmLZMA, 2)
        self.assertEqual(Foundation.NSDataCompressionAlgorithmZlib, 3)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultHasType(
            Foundation.NSData.rangeOfData_options_range_, Foundation.NSRange.__typestr__
        )
        self.assertArgHasType(
            Foundation.NSData.rangeOfData_options_range_,
            2,
            Foundation.NSRange.__typestr__,
        )

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertArgIsBlock(
            Foundation.NSData.enumerateByteRangesUsingBlock_,
            0,
            b"vn^v" + Foundation.NSRange.__typestr__ + b"o^Z",
        )
        data = Foundation.NSData.dataWithBytes_length_(rawBytes, len(rawBytes))

        lst = []

        def cb(buf, rng, done):
            lst.append((buf, rng, done))
            return False

        data.enumerateByteRangesUsingBlock_(cb)
        self.assertEqual(lst, [(rawBytes, Foundation.NSRange(0, len(rawBytes)), None)])

        self.assertArgIsBlock(
            Foundation.NSData.initWithBytesNoCopy_length_deallocator_,
            2,
            b"vn^v" + objc._C_NSUInteger,
        )

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertResultIsBOOL(
            Foundation.NSMutableData.decompressUsingAlgorithm_error_
        )
        self.assertArgIsOut(Foundation.NSMutableData.decompressUsingAlgorithm_error_, 1)

        self.assertResultIsBOOL(Foundation.NSMutableData.compressUsingAlgorithm_error_)
        self.assertArgIsOut(Foundation.NSMutableData.compressUsingAlgorithm_error_, 1)

    def assertDataContents(self, d1, d2, rawData):
        self.assertEqual(len(d1), d1.length(), "d1: len() and -length didn't match.")
        self.assertEqual(
            len(d1),
            len(rawData),
            "d1: len(<data>) and len(<input>) didn't match. %d vs %d"
            % (len(d1), len(rawData)),
        )
        self.assertEqual(len(d2), d2.length(), "d2: len() and -length didn't match.")
        self.assertEqual(
            len(d2),
            len(rawData),
            "d2: len(<data>) and len(<input>) didn't match. %d vs %d"
            % (len(d2), len(rawData)),
        )

    def testDataWithBytes_length_(self):
        # Test +dataWithBytes:length
        data = Foundation.NSData.dataWithBytes_length_(rawBytes, len(rawBytes))
        mutableData = Foundation.NSMutableData.dataWithBytes_length_(
            rawBytes, len(rawBytes)
        )
        self.assertDataContents(data, mutableData, rawBytes)

    def testAppendBytes_length_(self):
        self.assertArgIsIn(Foundation.NSMutableData.appendBytes_length_, 0)
        self.assertArgSizeInArg(Foundation.NSMutableData.appendBytes_length_, 0, 1)

    def testreplaceBytesInRange_withBytes_(self):
        self.assertArgIsIn(Foundation.NSMutableData.replaceBytesInRange_withBytes_, 1)
        self.assertArgSizeInArg(
            Foundation.NSMutableData.replaceBytesInRange_withBytes_, 1, 0
        )

    def testreplaceBytesInRange_withBytes_length_(self):
        self.assertArgIsIn(
            Foundation.NSMutableData.replaceBytesInRange_withBytes_length_, 1
        )
        self.assertArgSizeInArg(
            Foundation.NSMutableData.replaceBytesInRange_withBytes_length_, 1, 2
        )

    def testDataWithBytesNoCopy_length_freeWhenDone_(self):
        data = Foundation.NSData.dataWithBytesNoCopy_length_freeWhenDone_(
            rawBytes, len(rawBytes), False
        )
        mutableData = Foundation.NSMutableData.dataWithBytesNoCopy_length_freeWhenDone_(
            rawBytes, len(rawBytes), False
        )
        self.assertDataContents(data, mutableData, rawBytes)

    def testInitWithBytes_length_(self):
        # Test -initWithBytes:length:
        data = Foundation.NSData.alloc().initWithBytes_length_(rawBytes, len(rawBytes))
        mutableData = Foundation.NSMutableData.alloc().initWithBytes_length_(
            rawBytes, len(rawBytes)
        )
        self.assertDataContents(data, mutableData, rawBytes)

    def testInitWithBytesNoCopy_length_freeWhenDone_(self):
        # Test -initWithBytesNoCopy:length:
        data = Foundation.NSData.alloc().initWithBytesNoCopy_length_freeWhenDone_(
            rawBytes, len(rawBytes), False
        )
        mutableData = (
            Foundation.NSMutableData.alloc().initWithBytesNoCopy_length_freeWhenDone_(
                rawBytes, len(rawBytes), False
            )
        )
        self.assertDataContents(data, mutableData, rawBytes)

    def testBytes(self):
        # Test -bytes
        data = Foundation.NSData.alloc().initWithBytes_length_(rawBytes, len(rawBytes))
        bytesValue = data.bytes()
        self.assertEqual(
            len(bytesValue), len(rawBytes), "bytes() and rawBytes not equal length."
        )

        self.assertEqual(rawBytes, bytesValue)

        try:
            bytesValue[3] = b"\xAE"
        except TypeError as r:
            if str(r).find("buffer is read-only") == 0:
                pass
            elif str(r).find("cannot modify read-only memory") == 0:
                pass
            else:
                raise

    def testMutableBytes(self):
        # Test -mutableBytes
        mutableData = Foundation.NSMutableData.dataWithBytes_length_(
            rawBytes, len(rawBytes)
        )
        mutableBytes = mutableData.mutableBytes()
        for i in range(0, len(mutableBytes)):
            if sys.version_info[:2] >= (3, 3):
                mutableBytes[i] = (otherBytes[i : i + 1]).tobytes()[0]
            else:
                mutableBytes[i] = (otherBytes[i : i + 1]).tobytes()
        mutableBytes[1:8] = (otherBytes[1:8]).tobytes()

        try:
            mutableBytes[2:10] = (otherBytes[1:5]).tobytes()
        except (TypeError, ValueError) as r:
            if str(r).find("right operand length must match slice length") == 0:
                pass
            elif str(r).find("lvalue and rvalue have different structures") != -1:
                pass
            elif "cannot modify size of memoryview object" in str(r):
                pass
            elif (
                "ndarray assignment: lvalue and rvalue have different structures"
                in str(r)
            ):
                pass
            else:
                raise

    def testVariousDataLengths(self):
        # Test data of different lengths.
        #
        # Data of different lengths may be stored in different subclasses
        # within the class cluster.
        testFactor = list(range(1, 64)) + [1000, 10000, 1_000_000]
        for aFactor in testFactor:
            bigRawBytes = b"1234567890" * aFactor

            mutableData = Foundation.NSMutableData.dataWithBytes_length_(
                bigRawBytes, len(bigRawBytes)
            )
            data = Foundation.NSData.dataWithBytes_length_(
                bigRawBytes, len(bigRawBytes)
            )

            self.assertDataContents(data, mutableData, bigRawBytes)

            mutableBytes = mutableData.mutableBytes()
            bytes_value = data.bytes()

            self.assertEqual(len(bytes_value), data.length())
            self.assertEqual(len(mutableBytes), mutableData.length())
            self.assertEqual(bytes_value, mutableBytes)

            mutableBytes[0 : len(mutableBytes)] = bytes_value[0 : len(bytes_value)]

    def testInitWithContents(self):
        b, err = Foundation.NSData.alloc().initWithContentsOfFile_options_error_(
            "/etc/hosts", 0, None
        )
        self.assertIsInstance(b, Foundation.NSData)
        self.assertIs(err, None)
        b2, err = Foundation.NSData.alloc().initWithContentsOfFile_options_error_(
            "/etc/hosts.nosuchfile", 0, None
        )
        self.assertIs(b2, None)
        self.assertIsInstance(err, Foundation.NSError)
        url = Foundation.NSURL.fileURLWithPath_isDirectory_("/etc/hosts", False)
        b, err = Foundation.NSData.alloc().initWithContentsOfURL_options_error_(
            url, 0, None
        )
        self.assertIsInstance(b, Foundation.NSData)
        self.assertIs(err, None)
        url = Foundation.NSURL.fileURLWithPath_isDirectory_(
            "/etc/hosts.nosuchfile", False
        )
        b2, err = Foundation.NSData.alloc().initWithContentsOfURL_options_error_(
            url, 0, None
        )
        self.assertIs(b2, None)
        self.assertIsInstance(err, Foundation.NSError)


class MyData(Foundation.NSData):
    def dataWithBytes_length_(self, data, length):
        return ("data", data, length)


BYTES = "dummy bytes"


class MyData2(Foundation.NSData):
    def initWithBytes_length_(self, data, length):
        return ("init", data, length)

    def length(self):
        return 42

    def bytes(self):  # noqa: A003
        return BYTES


class MyData3(Foundation.NSData):
    def initWithBytes_length_(self, value, length):
        self._bytes = value
        self._length = length
        return self

    def bytes(self):  # noqa: A003
        return self._bytes

    def length(self):
        if hasattr(self, "_length"):
            return self._length
        return -1


class MyData4(Foundation.NSData):
    def initWithBytes_length_(self, value, length):
        return self

    def bytes(self):  # noqa: A003
        return None

    def length(self):
        return -1


class MyData5(Foundation.NSData):
    def initWithBytes_length_(self, value, length):
        return self

    def bytes(self):  # noqa: A003
        raise ValueError("No bytes available")

    def length(self):
        return -1


class TestMyData(TestCase):
    # 'initWithBytes:length:' and 'dataWithBytes:length:' have custom IMP's
    def testData(self):
        r = PyObjC_TestClass3.makeDataWithBytes_method_(MyData, 0)
        self.assertEqual(r, ("data", b"hello world", 11))

    def testInit(self):
        r = PyObjC_TestClass3.makeDataWithBytes_method_(MyData2, 1)
        self.assertEqual(r, ("init", b"hello world", 11))

    def testBytes(self):
        r = PyObjC_TestClass3.makeDataWithBytes_method_(MyData3, 1)
        b = PyObjC_TestClass3.getBytes_(r)

        # Check for memoryview
        if isinstance(b.bytes(), memoryview):
            self.assertEqual(b.bytes().tobytes(), b"hello world")
        else:
            self.assertEqual(bytes(b.bytes()), b"hello world")

        self.assertEqual(b.getBytes_length_(None, 4), b"hell")
        self.assertEqual(b.getBytes_range_(None, Foundation.NSRange(2, 4)), b"llo ")

    def testBytesNone(self):
        b = PyObjC_TestClass3.makeDataWithBytes_method_(MyData4, 1)
        self.assertEqual(b.bytes(), None)

    def testBytesRaises(self):
        b = PyObjC_TestClass3.makeDataWithBytes_method_(MyData5, 1)
        self.assertRaises(ValueError, b.bytes)


class TestBuffer(TestCase):
    def testArray(self):
        pool = Foundation.NSAutoreleasePool.alloc().init()
        a = array.array("b", b"foo")
        m = Foundation.NSMutableData.dataWithData_(a)
        self.assertEqual(a.tobytes(), m[:])
        self.assertTrue(objc.repythonify(a) is a)
        del pool
        a.frombytes(m)
        self.assertEqual(a.tobytes(), b"foofoo")
        m.appendData_(a)
        self.assertEqual(m[:], b"foofoofoo")
        m[3:6] = b"bar"
        self.assertEqual(m[:], b"foobarfoo")

    def testBuffer(self):
        b = b"foo"
        m = Foundation.NSMutableData.dataWithData_(b)
        self.assertEqual(b[:], m[:])
        self.assertTrue(objc.repythonify(b) is b)
        self.assertEqual(memoryview(m)[:], m[:])


class TestRegressions(TestCase):
    def testDataStr(self):
        input_bytes = b"hello"
        input_str = str(input_bytes)

        buf = Foundation.NSData.dataWithData_(input_bytes)
        self.assertEqual(str(buf), input_str)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSTableViewHelper(AppKit.NSObject):
    def tableView_rowActionsForRow_edge_(self, a, b, c):
        pass

    def tableView_viewForTableColumn_row_(self, a, b, c):
        pass

    def tableView_rowViewForRow_(self, a, b):
        pass

    def tableView_didAddRowView_forRow_(self, a, b, c):
        pass

    def tableView_didRemoveRowView_forRow_(self, a, b, c):
        pass

    def tableView_draggingSession_willBeginAtPoint_forRowIndexes_(self, a, b, c, d):
        pass

    def tableView_draggingSession_endedAtPoint_operation_(self, a, b, c, d):
        pass

    def numberOfRowsInTableView_(self, tv):
        return 1

    def tableView_objectValueForTableColumn_row_(self, tv, c, r):
        return 1

    def tableView_setObjectValue_forTableColumn_row_(self, o, tv, c, r):
        pass

    def tableView_writeRowsWithIndexes_toPasteboard_(self, tv, r, p):
        return 1

    def tableView_validateDrop_proposedRow_proposedDropOperation_(self, tv, dr, r, o):
        return 1

    def tableView_acceptDrop_row_dropOperation_(self, tv, dr, r, o):
        return 1

    def tableView_writeRows_toPasteboard_(self, tv, r, p):
        return 1

    def tableView_willDisplayCell_forTableColumn_row_(self, tv, c, tc, r):
        return 1

    def tableView_shouldEditTableColumn_row_(self, tv, tc, r):
        return 1

    def selectionShouldChangeInTableView_(self, tv):
        return 1

    def tableView_shouldSelectRow_(self, tv, r):
        return 1

    def tableView_shouldSelectTableColumn_(self, tv, tc):
        return 1

    def tableView_toolTipForCell_rect_tableColumn_row_mouseLocation_(
        self, tv, c, re, tc, r, x
    ):
        return 1

    def tableView_heightOfRow_(self, tv, r):
        return 1

    def tableView_typeSelectStringForTableColumn_row_(self, tv, tc, r):
        return 1

    def tableView_nextTypeSelectMatchFromRow_toRow_forString_(self, tv, r1, r2, s):
        return 1

    def tableView_shouldTypeSelectForEvent_withCurrentSearchString_(self, tv, e, s):
        return 1

    def tableView_shouldShowCellExpansionForTableColumn_row_(self, tv, tc, r):
        return 1

    def tableView_shouldTrackCell_forTableColumn_row_(self, tv, c, tc, r):
        return 1

    def tableView_dataCellForTableColumn_row_(self, tv, tc, r):
        return 1

    def tableView_isGroupRow_(self, tv, r):
        return 1

    def tableView_sizeToFitWidthOfColumn_(self, tv, c):
        return 1

    def tableView_shouldReorderColumn_toColumn_(self, tv, c1, c2):
        return 1

    def tableView_pasteboardWriterForRow_(self, tv, r):
        return 1


class TestNSTableView(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSTableViewDropOn, 0)
        self.assertEqual(AppKit.NSTableViewDropAbove, 1)

        self.assertEqual(AppKit.NSTableViewNoColumnAutoresizing, 0)
        self.assertEqual(AppKit.NSTableViewUniformColumnAutoresizingStyle, 1)
        self.assertEqual(AppKit.NSTableViewSequentialColumnAutoresizingStyle, 2)
        self.assertEqual(AppKit.NSTableViewReverseSequentialColumnAutoresizingStyle, 3)
        self.assertEqual(AppKit.NSTableViewLastColumnOnlyAutoresizingStyle, 4)
        self.assertEqual(AppKit.NSTableViewFirstColumnOnlyAutoresizingStyle, 5)

        self.assertEqual(AppKit.NSTableViewGridNone, 0)
        self.assertEqual(AppKit.NSTableViewSolidVerticalGridLineMask, 1 << 0)
        self.assertEqual(AppKit.NSTableViewSolidHorizontalGridLineMask, 1 << 1)

        self.assertEqual(AppKit.NSTableViewSelectionHighlightStyleRegular, 0)
        self.assertEqual(AppKit.NSTableViewSelectionHighlightStyleSourceList, 1)

        self.assertIsInstance(AppKit.NSTableViewSelectionDidChangeNotification, str)
        self.assertIsInstance(AppKit.NSTableViewColumnDidMoveNotification, str)
        self.assertIsInstance(AppKit.NSTableViewColumnDidResizeNotification, str)
        self.assertIsInstance(AppKit.NSTableViewSelectionIsChangingNotification, str)

        self.assertEqual(AppKit.NSTableViewStyleAutomatic, 0)
        self.assertEqual(AppKit.NSTableViewStyleFullWidth, 1)
        self.assertEqual(AppKit.NSTableViewStyleInset, 2)
        self.assertEqual(AppKit.NSTableViewStyleSourceList, 3)
        self.assertEqual(AppKit.NSTableViewStylePlain, 4)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(AppKit.NSTableViewSelectionHighlightStyleNone, -1)
        self.assertEqual(AppKit.NSTableViewDraggingDestinationFeedbackStyleNone, -1)
        self.assertEqual(AppKit.NSTableViewDraggingDestinationFeedbackStyleRegular, 0)
        self.assertEqual(
            AppKit.NSTableViewDraggingDestinationFeedbackStyleSourceList, 1
        )

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSTableViewDashedHorizontalGridLineMask, 1 << 3)

        self.assertEqual(AppKit.NSTableViewRowSizeStyleDefault, -1)
        self.assertEqual(AppKit.NSTableViewRowSizeStyleCustom, 0)
        self.assertEqual(AppKit.NSTableViewRowSizeStyleSmall, 1)
        self.assertEqual(AppKit.NSTableViewRowSizeStyleMedium, 2)
        self.assertEqual(AppKit.NSTableViewRowSizeStyleLarge, 3)

        self.assertEqual(AppKit.NSTableViewAnimationEffectNone, 0)
        self.assertEqual(AppKit.NSTableViewAnimationEffectFade, 1)
        self.assertEqual(AppKit.NSTableViewAnimationEffectGap, 2)
        self.assertEqual(AppKit.NSTableViewAnimationSlideUp, 0x10)
        self.assertEqual(AppKit.NSTableViewAnimationSlideDown, 0x20)
        self.assertEqual(AppKit.NSTableViewAnimationSlideLeft, 0x30)
        self.assertEqual(AppKit.NSTableViewAnimationSlideRight, 0x40)

        self.assertIsInstance(AppKit.NSTableViewRowViewKey, str)

        self.assertEqual(AppKit.NSTableViewDashedHorizontalGridLineMask, 1 << 3)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertEqual(AppKit.NSTableViewDraggingDestinationFeedbackStyleGap, 2)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(AppKit.NSTableRowActionEdgeLeading, 0)
        self.assertEqual(AppKit.NSTableRowActionEdgeTrailing, 1)

    def testMethods(self):
        self.assertArgIsBOOL(AppKit.NSTableView.setAllowsColumnReordering_, 0)
        self.assertResultIsBOOL(AppKit.NSTableView.allowsColumnReordering)
        self.assertArgIsBOOL(AppKit.NSTableView.setAllowsColumnResizing_, 0)
        self.assertResultIsBOOL(AppKit.NSTableView.allowsColumnResizing)
        self.assertArgIsBOOL(
            AppKit.NSTableView.setUsesAlternatingRowBackgroundColors_, 0
        )
        self.assertResultIsBOOL(AppKit.NSTableView.usesAlternatingRowBackgroundColors)
        self.assertArgIsBOOL(AppKit.NSTableView.setVerticalMotionCanBeginDrag_, 0)
        self.assertResultIsBOOL(AppKit.NSTableView.verticalMotionCanBeginDrag)
        self.assertResultIsBOOL(AppKit.NSTableView.canDragRowsWithIndexes_atPoint_)
        self.assertArgIsInOut(
            AppKit.NSTableView.dragImageForRowsWithIndexes_tableColumns_event_offset_, 3
        )
        self.assertArgIsBOOL(
            AppKit.NSTableView.setDraggingSourceOperationMask_forLocal_, 1
        )
        self.assertResultIsBOOL(AppKit.NSTableView.verticalMotionCanBeginDrag)
        self.assertArgIsBOOL(AppKit.NSTableView.setAllowsMultipleSelection_, 0)
        self.assertResultIsBOOL(AppKit.NSTableView.allowsMultipleSelection)
        self.assertArgIsBOOL(AppKit.NSTableView.setAllowsEmptySelection_, 0)
        self.assertResultIsBOOL(AppKit.NSTableView.allowsEmptySelection)
        self.assertArgIsBOOL(AppKit.NSTableView.setAllowsColumnSelection_, 0)
        self.assertResultIsBOOL(AppKit.NSTableView.allowsColumnSelection)
        self.assertArgIsBOOL(
            AppKit.NSTableView.selectColumnIndexes_byExtendingSelection_, 1
        )
        self.assertArgIsBOOL(
            AppKit.NSTableView.selectRowIndexes_byExtendingSelection_, 1
        )
        self.assertResultIsBOOL(AppKit.NSTableView.isColumnSelected_)
        self.assertResultIsBOOL(AppKit.NSTableView.isRowSelected_)
        self.assertResultIsBOOL(AppKit.NSTableView.textShouldBeginEditing_)
        self.assertResultIsBOOL(AppKit.NSTableView.textShouldEndEditing_)
        self.assertArgIsBOOL(AppKit.NSTableView.setAutosaveTableColumns_, 0)
        self.assertResultIsBOOL(AppKit.NSTableView.autosaveTableColumns)
        self.assertArgIsBOOL(AppKit.NSTableView.editColumn_row_withEvent_select_, 3)

        self.assertArgHasType(
            AppKit.NSTableView.drawBackgroundInClipRect_, 0, AppKit.NSRect.__typestr__
        )

        self.assertArgIsBOOL(AppKit.NSTableView.setDrawsGrid_, 0)
        self.assertResultIsBOOL(AppKit.NSTableView.drawsGrid)
        self.assertArgIsBOOL(AppKit.NSTableView.selectColumn_byExtendingSelection_, 1)
        self.assertArgIsBOOL(AppKit.NSTableView.selectRow_byExtendingSelection_, 1)
        self.assertArgIsInOut(
            AppKit.NSTableView.dragImageForRows_event_dragImageOffset_, 2
        )
        self.assertArgIsBOOL(AppKit.NSTableView.setAutoresizesAllColumnsToFit_, 0)
        self.assertResultIsBOOL(AppKit.NSTableView.autoresizesAllColumnsToFit)

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertArgIsBOOL(AppKit.NSTableView.setAllowsTypeSelect_, 0)
        self.assertResultIsBOOL(AppKit.NSTableView.allowsTypeSelect)

        self.assertArgHasType(
            AppKit.NSTableView.columnIndexesInRect_, 0, AppKit.NSRect.__typestr__
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(AppKit.NSTableView.shouldFocusCell_atColumn_row_)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsBOOL(AppKit.NSTableView.viewAtColumn_row_makeIfNecessary_, 2)
        self.assertArgIsBOOL(AppKit.NSTableView.rowViewAtRow_makeIfNecessary_, 1)

        self.assertArgIsBlock(
            AppKit.NSTableView.enumerateAvailableRowViewsUsingBlock_,
            0,
            b"v@" + objc._C_NSInteger,
        )

        self.assertResultIsBOOL(AppKit.NSTableView.floatsGroupRows)
        self.assertArgIsBOOL(AppKit.NSTableView.setFloatsGroupRows_, 0)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(AppKit.NSTableView.usesStaticContents)
        self.assertArgIsBOOL(AppKit.NSTableView.setUsesStaticContents_, 0)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(AppKit.NSTableView.rowActionsVisible)
        self.assertArgIsBOOL(AppKit.NSTableView.setRowActionsVisible_, 0)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(AppKit.NSTableView.usesAutomaticRowHeights)
        self.assertArgIsBOOL(AppKit.NSTableView.setUsesAutomaticRowHeights_, 0)

    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSTableViewDelegate")
        objc.protocolNamed("NSTableViewDataSource")

    def testProtocols(self):
        self.assertResultHasType(
            TestNSTableViewHelper.numberOfRowsInTableView_, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_objectValueForTableColumn_row_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_setObjectValue_forTableColumn_row_,
            3,
            objc._C_NSInteger,
        )
        self.assertResultIsBOOL(
            TestNSTableViewHelper.tableView_writeRowsWithIndexes_toPasteboard_
        )
        self.assertResultHasType(
            TestNSTableViewHelper.tableView_validateDrop_proposedRow_proposedDropOperation_,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_validateDrop_proposedRow_proposedDropOperation_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_validateDrop_proposedRow_proposedDropOperation_,
            3,
            objc._C_NSUInteger,
        )
        self.assertResultIsBOOL(
            TestNSTableViewHelper.tableView_acceptDrop_row_dropOperation_
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_acceptDrop_row_dropOperation_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_acceptDrop_row_dropOperation_,
            3,
            objc._C_NSUInteger,
        )
        self.assertResultIsBOOL(TestNSTableViewHelper.tableView_writeRows_toPasteboard_)

        self.assertArgHasType(
            TestNSTableViewHelper.tableView_willDisplayCell_forTableColumn_row_,
            3,
            objc._C_NSInteger,
        )
        self.assertResultIsBOOL(
            TestNSTableViewHelper.tableView_shouldEditTableColumn_row_
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_shouldEditTableColumn_row_,
            2,
            objc._C_NSInteger,
        )
        self.assertResultIsBOOL(TestNSTableViewHelper.selectionShouldChangeInTableView_)
        self.assertResultIsBOOL(TestNSTableViewHelper.tableView_shouldSelectRow_)
        self.assertResultIsBOOL(
            TestNSTableViewHelper.tableView_shouldSelectTableColumn_
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_toolTipForCell_rect_tableColumn_row_mouseLocation_,
            2,
            b"N^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_toolTipForCell_rect_tableColumn_row_mouseLocation_,
            4,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_toolTipForCell_rect_tableColumn_row_mouseLocation_,
            5,
            AppKit.NSPoint.__typestr__,
        )
        self.assertResultHasType(
            TestNSTableViewHelper.tableView_heightOfRow_, objc._C_CGFloat
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_heightOfRow_, 1, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_typeSelectStringForTableColumn_row_,
            2,
            objc._C_NSInteger,
        )
        self.assertResultHasType(
            TestNSTableViewHelper.tableView_nextTypeSelectMatchFromRow_toRow_forString_,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_nextTypeSelectMatchFromRow_toRow_forString_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_nextTypeSelectMatchFromRow_toRow_forString_,
            2,
            objc._C_NSInteger,
        )
        self.assertResultIsBOOL(
            TestNSTableViewHelper.tableView_shouldTypeSelectForEvent_withCurrentSearchString_
        )
        self.assertResultIsBOOL(
            TestNSTableViewHelper.tableView_shouldShowCellExpansionForTableColumn_row_
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_shouldShowCellExpansionForTableColumn_row_,
            2,
            objc._C_NSInteger,
        )
        self.assertResultIsBOOL(
            TestNSTableViewHelper.tableView_shouldTrackCell_forTableColumn_row_
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_shouldTrackCell_forTableColumn_row_,
            3,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_dataCellForTableColumn_row_,
            2,
            objc._C_NSInteger,
        )
        self.assertResultIsBOOL(TestNSTableViewHelper.tableView_isGroupRow_)
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_isGroupRow_, 1, objc._C_NSInteger
        )

    @min_os_level("10.6")
    def testProtocols10_6(self):
        self.assertResultHasType(
            TestNSTableViewHelper.tableView_sizeToFitWidthOfColumn_, objc._C_CGFloat
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_sizeToFitWidthOfColumn_,
            1,
            objc._C_NSInteger,
        )

        self.assertResultIsBOOL(
            TestNSTableViewHelper.tableView_shouldReorderColumn_toColumn_
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_shouldReorderColumn_toColumn_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_shouldReorderColumn_toColumn_,
            2,
            objc._C_NSInteger,
        )

    @min_os_level("10.7")
    def testProtocols10_7(self):
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_pasteboardWriterForRow_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_viewForTableColumn_row_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_rowViewForRow_, 1, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_didAddRowView_forRow_, 2, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_didRemoveRowView_forRow_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_draggingSession_willBeginAtPoint_forRowIndexes_,
            2,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_draggingSession_endedAtPoint_operation_,
            2,
            AppKit.NSPoint.__typestr__,
        )

    @min_os_level("10.11")
    def testProtococols10_11(self):
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_rowActionsForRow_edge_, 1, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSTableViewHelper.tableView_rowActionsForRow_edge_, 2, objc._C_NSInteger
        )



import Cocoa
import objc
from ToDoDocument import ToDoDocument
from ToDoItem import (
    COMPLETE,
    SECS_IN_HOUR,
    SECS_IN_DAY,
    ToDoItem,
    ConvertSecondsToTime,
    ConvertTimeToSeconds,
    ToDoItemChangedNotification,
)

NOTIFY_TAG = 0
RESCHEDULE_TAG = 1
NOTES_TAG = 2

NotifyLengthNone = 0
NotifyLengthQuarter = 1
NotifyLengthHour = 2
NotifyLengthDay = 3
NotifyLengthOther = 4

_sharedInfoWindowController = None


class InfoWindowController(Cocoa.NSWindowController):
    dummyView = objc.IBOutlet()
    infoDate = objc.IBOutlet()
    infoItem = objc.IBOutlet()
    infoNotes = objc.IBOutlet()
    infoNotifyAMPM = objc.IBOutlet()
    infoNotifyHour = objc.IBOutlet()
    infoNotifyMinute = objc.IBOutlet()
    infoNotifyOtherHours = objc.IBOutlet()
    infoNotifySwitchMatrix = objc.IBOutlet()
    infoPopUp = objc.IBOutlet()
    infoSchedComplete = objc.IBOutlet()
    infoSchedDate = objc.IBOutlet()
    infoSchedMatrix = objc.IBOutlet()
    infoWindowViews = objc.IBOutlet()
    notesView = objc.IBOutlet()
    notifyView = objc.IBOutlet()
    reschedView = objc.IBOutlet()

    __slots__ = ("_inspectingDocument",)

    @objc.IBAction
    def switchClicked_(self, sender):
        dueSecs = 0
        idx = 0
        theItem = self._inspectingDocument.selectedItem()
        if theItem is None:
            return

        if sender is self.infoNotifyAMPM:
            if self.infoNotifyHour.intValue():
                pmFlag = self.infoNotifyAMPM.selectedRow() == 1
                dueSecs = ConvertTimeToSeconds(
                    self.infoNotifyHour.intValue(),
                    self.infoNotifyMinute.intValue(),
                    pmFlag,
                )
                theItem.setSecsUntilDue_(dueSecs)
        elif sender is self.infoNotifySwitchMatrix:
            idx = self.infoNotifySwitchMatrix.selectedRow()

            if not theItem:
                pass
            elif idx == NotifyLengthNone:
                theItem.setSecsUntilNotify_(0)
            elif idx == NotifyLengthQuarter:
                theItem.setSecsUntilNotify_(SECS_IN_HOUR / 4)
            elif idx == NotifyLengthHour:
                theItem.setSecsUntilNotify_(SECS_IN_HOUR)
            elif idx == NotifyLengthDay:
                theItem.setSecsUntilNotify_(SECS_IN_DAY)
            elif idx == NotifyLengthOther:
                theItem.setSecsUntilNotify_(
                    self.infoNotifyOtherHours.intValue() * SECS_IN_HOUR
                )
            else:
                Cocoa.NSLog("Error in selectedRow")
        elif sender is self.infoSchedComplete:
            if theItem:
                theItem.setStatus_(COMPLETE)
        elif sender is self.infoSchedMatrix:
            # left as an exercise in the objective-C code
            pass

        self.updateInfoWindow()
        self._inspectingDocument.selectedItemModified()

    def textDidChange_(self, notification):
        if notification.object() is self.infoNotes:
            self._inspectingDocument.selectedItem().setNotes_(self.infoNotes.string())
            self._inspectingDocument.selectItemModified()

    def textDidEndEditing_(self, notification):
        if notification.object() is self.infoNotes:
            self._inspectingDocument.selectedItem().setNotes_(self.infoNotes.string())
            self._inspectingDocument.selectedItemModified()

    def controlTextDidEndEditing_(self, notification):
        dueSecs = 0
        theItem = self._inspectingDocument.selectedItem()
        if theItem is None:
            return

        if (notification.object() is self.infoNotifyHour) or (
            notification.object() is self.infoNotifyMinute
        ):

            dueSecs = ConvertTimeToSeconds(
                self.infoNotifyHour.intValue(),
                self.infoNotifyMinute.intValue(),
                self.infoNotifyAMPM.cellAtRow_column_(1, 0).state(),
            )
            theItem.setSecsUntilNotify_(dueSecs)
        elif notification.object() is self.infoNotifyOtherHours:
            if self.infoNotifySwitchMatrix.selectedRow() == NotifyLengthOther:
                theItem.setSecsUntilNotify_(
                    self.infoNotifyOtherHours.intValue() * SECS_IN_HOUR
                )
            else:
                return
        elif notification.object() is self.infoSchedDate:
            # Left as an exercise
            pass

        self._inspectingDocument.selectedItemModified()

    @classmethod
    def sharedInfoWindowController(self):
        global _sharedInfoWindowController

        if not _sharedInfoWindowController:
            _sharedInfoWindowController = InfoWindowController.alloc().init()

        return _sharedInfoWindowController

    def init(self):
        self = self.initWithWindowNibName_("ToDoInfoWindow")
        if self:
            self.setWindowFrameAutosaveName_("Info")

        return self

    def dump_outlets(self):
        print("dummyView", self.dummyView)
        print("infoDate", self.infoDate)
        print("infoItem", self.infoItem)
        print("infoNotes", self.infoNotes)
        print("infoNotifyAMPM", self.infoNotifyAMPM)
        print("infoNotifyHour", self.infoNotifyHour)
        print("infoNotifyMinute", self.infoNotifyMinute)
        print("infoNotifyOtherHours", self.infoNotifyOtherHours)
        print("infoNotifySwitchMatrix", self.infoNotifySwitchMatrix)
        print("infoPopUp", self.infoPopUp)
        print("infoSchedComplet", self.infoSchedComplete)
        print("infoSchedDate", self.infoSchedDate)
        print("infoSchedMatrix", self.infoSchedMatrix)
        print("infoWindowViews", self.infoWindowViews)
        print("notesView", self.notesView)
        print("notifyView", self.notifyView)
        print("reschedView", self.reschedView)

    def windowDidLoad(self):
        Cocoa.NSWindowController.windowDidLoad(self)

        self.notifyView.retain()
        self.notifyView.removeFromSuperview()

        self.reschedView.retain()
        self.reschedView.removeFromSuperview()

        self.notesView.retain()
        self.notesView.removeFromSuperview()

        self.infoWindowViews = None

        self.infoNotes.setDelegate_(self)
        self.swapInfoWindowView_(self)
        self.setMainWindow_(Cocoa.NSApp().mainWindow())
        self.updateInfoWindow()

        Cocoa.NSNotificationCenter.defaultCenter().addObserver_selector_name_object_(
            self, "mainWindowChanged:", Cocoa.NSWindowDidBecomeMainNotification, None
        )

        Cocoa.NSNotificationCenter.defaultCenter().addObserver_selector_name_object_(
            self, "mainWindowResigned:", Cocoa.NSWindowDidResignMainNotification, None
        )

        Cocoa.NSNotificationCenter.defaultCenter().addObserver_selector_name_object_(
            self, "selectedItemChanged:", ToDoItemChangedNotification, None
        )

    def __del__(self):  # dealloc

        Cocoa.NSNotificationCenter.defaultCenter().removeObserver_(self)

        # Cannot to this
        Cocoa.NSWindowController.dealloc(self)

    def updateInfoWindow(self):
        minute = 0
        hour = 0

        selected = self.infoPopUp.selectedItem().tag()
        selectedItem = self._inspectingDocument.selectedItem()

        if isinstance(selectedItem, ToDoItem):
            self.infoItem.setStringValue_(selectedItem.itemName())
            self.infoDate.setStringValue_(
                selectedItem.day().descriptionWithCalendarFormat_timeZone_locale_(
                    "%a, %b %d %Y", Cocoa.NSTimeZone.localTimeZone(), None
                )
            )

            if selected == NOTIFY_TAG:
                dueSecs = selectedItem.secsUntilDue()
                hour, minutes, pmFlag = ConvertSecondsToTime(dueSecs)
                self.infoNotifyAMPM.cellAtRow_column_(0, 0).setState_(not pmFlag)
                self.infoNotifyAMPM.cellAtRow_column_(1, 0).setState_(pmFlag)
                self.infoNotifyHour.setIntValue_(hour)
                self.infoNotifyMinute.setIntValue_(minute)

                notifySecs = selectedItem.secsUntilNotify()
                clearButtonMatrix(self.infoNotifySwitchMatrix)

                if notifySecs == 0:
                    self.infoNotifySwitchMatrix.cellAtRow_column_(
                        NotifyLengthNone, 0
                    ).setState_(Cocoa.NSOnState)
                elif notifySecs == SECS_IN_HOUR / 4:
                    self.infoNotifySwitchMatrix.cellAtRow_column_(
                        NotifyLengthQuarter, 0
                    ).setState_(Cocoa.NSOnState)
                elif notifySecs == SECS_IN_HOUR:
                    self.infoNotifySwitchMatrix.cellAtRow_column_(
                        NotifyLengthHour, 0
                    ).setState_(Cocoa.NSOnState)
                elif notifySecs == SECS_IN_DAY:
                    self.infoNotifySwitchMatrix.cellAtRow_column_(
                        NotifyLengthDay, 0
                    ).setState_(Cocoa.NSOnState)
                else:
                    self.infoNotifySwitchMatrix.cellAtRow_column_(
                        NotifyLengthOther, 0
                    ).setState_(Cocoa.NSOnState)
                    self.infoNotifyOtherHours.setIntValue_(notifySecs / SECS_IN_HOUR)
            elif selected == RESCHEDULE_TAG:
                # left as an exercise
                pass
            elif selected == NOTES_TAG:
                self.infoNotes.setString_(selectedItem.notes())
        else:
            self.infoItem.setStringValue_("")
            self.infoDate.setStringValue_("")
            self.infoNotifyHour.setStringValue_("")
            self.infoNotifyMinute.setStringValue_("")
            self.infoNotifyAMPM.cellAtRow_column_(0, 0).setState_(Cocoa.NSOnState)
            self.infoNotifyAMPM.cellAtRow_column_(1, 0).setState_(Cocoa.NSOffState)
            clearButtonMatrix(self.infoNotifySwitchMatrix)
            self.infoNotifySwitchMatrix.cellAtRow_column_(
                NotifyLengthNone, 0
            ).setState_(Cocoa.NSOnState)
            self.infoNotifyOtherHours.setStringValue_("")
            self.infoNotes.setString_("")

    def setMainWindow_(self, mainWindow):
        if not mainWindow:
            return

        controller = mainWindow.windowController()

        if isinstance(controller.document(), ToDoDocument):
            self._inspectingDocument = controller.document()
        else:
            self._inspectingDocument = None

        self.updateInfoWindow()

    def mainWindowChanged_(self, notification):
        self.setMainWindow_(notification.object())

    def mainWindowResigned_(self, notification):
        self.setMainWindow_(None)

    @objc.IBAction
    def swapInfoWindowView_(self, sender):
        selected = self.infoPopUp.selectedItem().tag()

        if selected == NOTIFY_TAG:
            newView = self.notifyView
        elif selected == RESCHEDULE_TAG:
            newView = self.reschedView
        elif selected == NOTES_TAG:
            newView = self.notesView

        if self.dummyView.contentView() != newView:
            self.dummyView.setContentView_(newView)

    def selectedItemChanged_(self, notification):
        self.updateInfoWindow()


def clearButtonMatrix(matrix):
    rows, cols = matrix.getNumberOfRows_columns_()

    for i in range(rows):
        cell = matrix.cellAtRow_column_(i, 0)
        if cell:
            cell.setState_(False)



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase


class TestNSURLHandle(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSHTTPPropertyStatusCodeKey, str)
        self.assertIsInstance(Foundation.NSHTTPPropertyStatusReasonKey, str)
        self.assertIsInstance(Foundation.NSHTTPPropertyServerHTTPVersionKey, str)
        self.assertIsInstance(Foundation.NSHTTPPropertyRedirectionHeadersKey, str)
        self.assertIsInstance(Foundation.NSHTTPPropertyErrorPageDataKey, str)
        self.assertIsInstance(Foundation.NSHTTPPropertyHTTPProxy, str)
        self.assertIsInstance(Foundation.NSFTPPropertyUserLoginKey, str)
        self.assertIsInstance(Foundation.NSFTPPropertyUserPasswordKey, str)
        self.assertIsInstance(Foundation.NSFTPPropertyActiveTransferModeKey, str)
        self.assertIsInstance(Foundation.NSFTPPropertyFileOffsetKey, str)
        self.assertIsInstance(Foundation.NSFTPPropertyFTPProxy, str)

        self.assertEqual(Foundation.NSURLHandleNotLoaded, 0)
        self.assertEqual(Foundation.NSURLHandleLoadSucceeded, 1)
        self.assertEqual(Foundation.NSURLHandleLoadInProgress, 2)
        self.assertEqual(Foundation.NSURLHandleLoadFailed, 3)

    def testMethods(self):
        self.assertArgIsBOOL(Foundation.NSURLHandle.didLoadBytes_loadComplete_, 1)
        self.assertResultIsBOOL(Foundation.NSURLHandle.canInitWithURL_)
        self.assertArgIsBOOL(Foundation.NSURLHandle.initWithURL_cached_, 1)
        self.assertResultIsBOOL(Foundation.NSURLHandle.writeProperty_forKey_)
        self.assertResultIsBOOL(Foundation.NSURLHandle.writeData_)

    def testProtocols(self):
        objc.protocolNamed("NSURLHandleClient")



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSStatusBar(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSVariableStatusItemLength, -1.0)
        self.assertEqual(AppKit.NSSquareStatusItemLength, -2.0)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSStatusBar.isVertical)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSRunningApplication(TestCase):
    @min_os_level("10.6")
    def testConstants(self):
        self.assertEqual(AppKit.NSApplicationActivateAllWindows, 1 << 0)
        self.assertEqual(AppKit.NSApplicationActivateIgnoringOtherApps, 1 << 1)

        self.assertEqual(AppKit.NSApplicationActivationPolicyRegular, 0)
        self.assertEqual(AppKit.NSApplicationActivationPolicyAccessory, 1)
        self.assertEqual(AppKit.NSApplicationActivationPolicyProhibited, 2)

    @min_os_level("10.6")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSRunningApplication.isTerminated)
        self.assertResultIsBOOL(AppKit.NSRunningApplication.isFinishedLaunching)
        self.assertResultIsBOOL(AppKit.NSRunningApplication.isHidden)
        self.assertResultIsBOOL(AppKit.NSRunningApplication.isActive)
        self.assertResultIsBOOL(AppKit.NSRunningApplication.hide)
        self.assertResultIsBOOL(AppKit.NSRunningApplication.unhide)
        self.assertResultIsBOOL(AppKit.NSRunningApplication.activateWithOptions_)
        self.assertResultIsBOOL(AppKit.NSRunningApplication.terminate)
        self.assertResultIsBOOL(AppKit.NSRunningApplication.forceTerminate)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSRunningApplication.ownsMenuBar)



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSURLProtocolHelper(Foundation.NSObject):
    def URLProtocol_didReceiveResponse_cacheStoragePolicy_(self, a, b, c):
        pass


class TestNSURLProtocol(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSURLProtocol.canInitWithRequest_)
        self.assertResultIsBOOL(
            Foundation.NSURLProtocol.requestIsCacheEquivalent_toRequest_
        )
        self.assertResultIsBOOL(Foundation.NSURLProtocol.registerClass_)

        self.assertArgHasType(
            TestNSURLProtocolHelper.URLProtocol_didReceiveResponse_cacheStoragePolicy_,
            2,
            objc._C_NSUInteger,
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(Foundation.NSURLProtocol.canInitWithTask_)

    def testProtocols(self):
        objc.protocolNamed("NSURLProtocolClient")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSButtonTouchBarItem(TestCase):
    @min_os_level("10.15")
    def test_methods(self):
        self.assertArgIsSEL(
            AppKit.NSButtonTouchBarItem.buttonTouchBarItemWithIdentifier_title_target_action_,
            3,
            b"v@:@",
        )
        self.assertArgIsSEL(
            AppKit.NSButtonTouchBarItem.buttonTouchBarItemWithIdentifier_image_target_action_,
            3,
            b"v@:@",
        )
        self.assertArgIsSEL(
            AppKit.NSButtonTouchBarItem.buttonTouchBarItemWithIdentifier_title_image_target_action_,  # noqa: B950
            4,
            b"v@:@",
        )



import Cocoa
import objc

# date/time element popup selections:
kNSHourMinuteDatePickerElementFlag = 0
kNSHourMinuteSecondDatePickerElementFlag = 1
kNSTimeZoneDatePickerElementFlag = 2

kNSYearMonthDatePickerElementFlag = 0
kNSYearMonthDayDatePickerElementFlag = 1
kNSEraDatePickerElementFlag = 2

kSingleDateMode = 0
kRangeDateMode = 1


class MyWindowController(Cocoa.NSWindowController):
    datePickerControl = objc.ivar()
    shrinkGrowFactor = objc.ivar.int()

    outerBox = objc.IBOutlet()
    datePickerPlaceHolder = objc.IBOutlet()

    dateResult1 = objc.IBOutlet()
    dateResult2 = objc.IBOutlet()
    dateResult3 = objc.IBOutlet()
    dateResult4 = objc.IBOutlet()
    dateResult5 = objc.IBOutlet()

    # appearance
    pickerStylePopup = objc.IBOutlet()
    drawsBackgroundCheck = objc.IBOutlet()
    bezeledCheck = objc.IBOutlet()
    borderedCheck = objc.IBOutlet()
    backColorWell = objc.IBOutlet()
    textColorWell = objc.IBOutlet()
    fontSizePopup = objc.IBOutlet()

    # date and time
    dateElementChecks = objc.IBOutlet()
    timeElementChecks = objc.IBOutlet()
    overrideDateCheck = objc.IBOutlet()
    overrideDate = objc.IBOutlet()

    # date range
    datePickerModeRadios = objc.IBOutlet()
    secondsRangeEdit = objc.IBOutlet()
    secondsRangeEditLabel = objc.IBOutlet()

    minDatePicker = objc.IBOutlet()
    maxDatePicker = objc.IBOutlet()
    setMinDateButton = objc.IBOutlet()
    setMaxDateButton = objc.IBOutlet()
    clearMinDateButton = objc.IBOutlet()
    clearMaxDateButton = objc.IBOutlet()

    def awakeFromNib(self):
        # based our date formatter on CFDateFormatter: allows more
        # configurability and better localization
        Cocoa.NSDateFormatter.setDefaultFormatterBehavior_(
            Cocoa.NSDateFormatterBehavior10_4
        )

        self.setupDatePickerControl_(Cocoa.NSClockAndCalendarDatePickerStyle)

        # setup the initial Cocoa.NSDatePickerElementFlags since we are using picker style:
        # Cocoa.NSClockAndCalendarDatePickerStyle
        flags = (
            Cocoa.NSYearMonthDatePickerElementFlag
            | Cocoa.NSYearMonthDayDatePickerElementFlag
            | Cocoa.NSEraDatePickerElementFlag
            | Cocoa.NSHourMinuteDatePickerElementFlag
            | Cocoa.NSHourMinuteSecondDatePickerElementFlag
            | Cocoa.NSTimeZoneDatePickerElementFlag
        )
        self.datePickerControl.setDatePickerElements_(flags)

        self.datePickerModeRadios.cellWithTag_(1).setEnabled_(
            False
        )  # not currently implemented in 10.4.x and earlier

        self.minDatePicker.setDateValue_(Cocoa.NSDate.date())
        self.maxDatePicker.setDateValue_(Cocoa.NSDate.distantFuture())

        self.updateControls()  # force update of all UI elements and the picker itself

    def applicationShouldTerminateAfterLastWindowClosed_(self, sender):
        return True

    def setupDatePickerControl_(self, pickerStyle):
        # we need to re-create the picker control (due to a resize bug when
        # switching between styles)
        if (
            self.datePickerControl is not None
        ):  # hide and release the previous date picker, if any
            self.datePickerControl.setHidden_(True)
            self.datePickerControl.setNeedsDisplay_(True)
            self.datePickerControl = None

        frame = self.datePickerPlaceHolder.frame()
        self.shrinkGrowFactor = frame.size.height - 30

        # create the date picker control if not created already
        if self.datePickerControl is None:
            self.datePickerControl = Cocoa.NSDatePicker.alloc().initWithFrame_(frame)

        self.datePickerControl.setFrameOrigin_(
            Cocoa.NSMakePoint(1, 0)
        )  # nudge the control placement a little

        self.datePickerControl.setDatePickerStyle_(
            pickerStyle
        )  # set our desired picker style

        self.datePickerPlaceHolder.addSubview_(
            self.datePickerControl
        )  # embed into our placeholder
        self.datePickerControl.setDrawsBackground_(True)
        self.datePickerControl.setBezeled_(True)
        self.datePickerControl.setBordered_(False)
        self.datePickerControl.setEnabled_(True)

        self.datePickerControl.setTextColor_(self.textColorWell.color())
        self.datePickerControl.setBackgroundColor_(self.backColorWell.color())

        # always set the date/time to TODAY
        # note that our delete override might block this...
        self.datePickerControl.setDateValue_(Cocoa.NSDate.date())

        self.datePickerControl.setNeedsDisplay_(True)
        self.updateControls()  # force update of all UI elements and the picker itself

        # synch the picker style popup with the new style change
        self.pickerStylePopup.selectItemWithTag_(pickerStyle)

        # we want to be the cell's delegate to catch date validation
        self.datePickerControl.setDelegate_(self)
        # or we can set us as the delegate to its cell like so:
        #                self.datePickerControl.cell().setDelegate_(self)

        # we want to respond to date/time changes
        self.datePickerControl.setAction_("datePickerAction:")

    def updateDateResult(self):
        theDate = self.datePickerControl.dateValue()
        if theDate is not None:
            formatter = Cocoa.NSDateFormatter.alloc().init()

            # some examples:
            # formatter.setDateStyle_(Cocoa.NSDateFormatterNoStyle)
            #     ->  <no date displayed>
            # formatter.setDateStyle_(Cocoa.NSDateFormatterMediumStyle)
            #     ->  Jan 24, 1984
            # formatter.setDateStyle_(Cocoa.NSDateFormatterShortStyle)
            #     ->  1/24/84
            # formatter.setDateStyle_(Cocoa.NSDateFormatterLongStyle)
            #     ->  January 24, 1984
            # formatter.setDateStyle_(Cocoa.NSDateFormatterFullStyle)
            #     ->  Tuesday, January 24, 1984
            #
            # formatter.setTimeStyle_(Cocoa.NSDateFormatterNoStyle)
            #     ->  <no time displayed>
            # formatter.setTimeStyle_(Cocoa.NSDateFormatterShortStyle)
            #     ->  2:44 PM
            # formatter.setTimeStyle_(Cocoa.NSDateFormatterMediumStyle)
            #     ->  2:44:55 PM
            # formatter.setTimeStyle_(Cocoa.NSDateFormatterLongStyle)
            #     ->  2:44:55 PM PDT
            # formatter.setTimeStyle_(Cocoa.NSDateFormatterFullStyle)
            #     ->  2:44:55 PM PDT
            #

            formatter.setDateStyle_(Cocoa.NSDateFormatterShortStyle)
            formatter.setTimeStyle_(Cocoa.NSDateFormatterNoStyle)
            formattedDateString = formatter.stringFromDate_(theDate)
            self.dateResult1.setStringValue_(formattedDateString)

            formatter.setDateStyle_(Cocoa.NSDateFormatterShortStyle)
            formatter.setTimeStyle_(Cocoa.NSDateFormatterShortStyle)
            formattedDateString = formatter.stringFromDate_(theDate)
            self.dateResult2.setStringValue_(formattedDateString)

            formatter.setDateStyle_(Cocoa.NSDateFormatterMediumStyle)
            formatter.setTimeStyle_(Cocoa.NSDateFormatterShortStyle)
            formattedDateString = formatter.stringFromDate_(theDate)
            self.dateResult3.setStringValue_(formattedDateString)

            formatter.setDateStyle_(Cocoa.NSDateFormatterLongStyle)
            formatter.setTimeStyle_(Cocoa.NSDateFormatterShortStyle)
            formattedDateString = formatter.stringFromDate_(theDate)
            self.dateResult4.setStringValue_(formattedDateString)

            formatter.setDateStyle_(Cocoa.NSDateFormatterFullStyle)
            formatter.setTimeStyle_(Cocoa.NSDateFormatterFullStyle)
            formattedDateString = formatter.stringFromDate_(theDate)
            self.dateResult5.setStringValue_(formattedDateString)

    def updateControls(self):
        self.datePickerControl.setNeedsDisplay_(True)  # force it to update

        self.updateDatePickerMode()
        self.updateDateTimeElementFlags()
        self.updateDateResult()

    @objc.IBAction
    def setPickerStyle_(self, sender):
        tag = sender.selectedCell().tag()

        windowFrame = self.window().frame()
        boxFrame = self.outerBox.frame()

        self.datePickerControl.setHidden_(True)

        if tag == Cocoa.NSClockAndCalendarDatePickerStyle:
            # for this picker style, we need to grow the window to make room

            windowFrame.size.height += self.shrinkGrowFactor
            windowFrame.origin.y -= self.shrinkGrowFactor

            boxFrame.size.height += self.shrinkGrowFactor
            self.outerBox.setFrame_(boxFrame)

            self.window().setFrame_display_animate_(windowFrame, True, True)

            # set our desired picker style
            self.datePickerControl.setDatePickerStyle_(
                Cocoa.NSClockAndCalendarDatePickerStyle
            )

            # shows these last
            self.dateResult1.setHidden_(False)
            self.dateResult2.setHidden_(False)
            self.dateResult3.setHidden_(False)
            self.dateResult4.setHidden_(False)

        else:
            currentPickerStyle = self.datePickerControl.datePickerStyle()

            # shrink the window only if the current style is "clock and calendar"
            if currentPickerStyle == Cocoa.NSClockAndCalendarDatePickerStyle:
                # hide these first
                self.dateResult1.setHidden_(True)
                self.dateResult2.setHidden_(True)
                self.dateResult3.setHidden_(True)
                self.dateResult4.setHidden_(True)

                windowFrame.size.height -= self.shrinkGrowFactor
                windowFrame.origin.y += self.shrinkGrowFactor

                boxFrame.size.height -= self.shrinkGrowFactor
                self.outerBox.setFrame_(boxFrame)

                self.window().setFrame_display_animate_(windowFrame, True, True)

            self.setupDatePickerControl_(tag)  # set our desired picker style

        self.datePickerControl.setHidden_(False)

        self.updateControls()  # force update of all UI elements and the picker itself

    @objc.IBAction
    def setFontSize_(self, sender):
        tag = sender.selectedCell().tag()
        if tag == Cocoa.NSMiniControlSize:
            self.datePickerControl.cell().setControlSize_(Cocoa.NSMiniControlSize)
            self.datePickerControl.cell().setFont_(Cocoa.NSFont.systemFontOfSize_(9.0))

        elif tag == Cocoa.NSSmallControlSize:
            self.datePickerControl.cell().setControlSize_(Cocoa.NSSmallControlSize)
            self.datePickerControl.cell().setFont_(Cocoa.NSFont.systemFontOfSize_(11.0))

        elif tag == Cocoa.NSRegularControlSize:
            self.datePickerControl.cell().setControlSize_(Cocoa.NSRegularControlSize)
            self.datePickerControl.cell().setFont_(Cocoa.NSFont.systemFontOfSize_(13.0))

    @objc.IBAction
    def datePickerAction_(self, sender):
        self.updateDateResult()

    @objc.IBAction
    def dateOverrideAction_(self, sender):
        checked = sender.selectedCell().state()
        if checked:
            self.datePickerControl.setDelegate_(self)

        else:
            self.datePickerControl.setDelegate_(None)

        self.datePickerControl.setDateValue_(
            Cocoa.NSDate.date()
        )  # force the delete "datePickerCell" to be called

    def datePickerCell_validateProposedDateValue_timeInterval_(
        self, aDatePickerCell, proposedDateValue, proposedTimeInterval
    ):
        controller = aDatePickerCell.delegate()

        if controller is self and aDatePickerCell is self.datePickerControl.cell():
            # override the date and time?
            if self.overrideDateCheck.cell().state():
                # override the date using the user specified date
                return (self.overrideDate.dateValue(), proposedTimeInterval)

        return (proposedDateValue, proposedTimeInterval)

    @objc.IBAction
    def setDrawsBackground_(self, sender):
        self.datePickerControl.setDrawsBackground_(sender.state())

    @objc.IBAction
    def setBackgroundColor_(self, sender):
        newColor = sender.color()
        self.datePickerControl.setBackgroundColor_(newColor)

    @objc.IBAction
    def setTextColor_(self, sender):
        newColor = sender.color()
        self.datePickerControl.setTextColor_(newColor)

    @objc.IBAction
    def setBezeled_(self, sender):
        self.datePickerControl.setBezeled_(sender.state())

    @objc.IBAction
    def setBordered_(self, sender):
        self.datePickerControl.setBordered_(sender.state())

    @objc.IBAction
    def setDateElementFlags_(self, sender):
        tag = sender.selectedCell().tag()
        flags = self.datePickerControl.datePickerElements()

        checked = sender.selectedCell().state()

        if tag == kNSYearMonthDatePickerElementFlag:
            if checked:
                flags |= Cocoa.NSYearMonthDatePickerElementFlag
            else:
                flags ^= Cocoa.NSYearMonthDatePickerElementFlag

        elif tag == kNSYearMonthDayDatePickerElementFlag:
            if checked:
                flags |= Cocoa.NSYearMonthDayDatePickerElementFlag
            else:
                flags ^= Cocoa.NSYearMonthDayDatePickerElementFlag

        elif tag == kNSEraDatePickerElementFlag:
            if checked:
                flags |= Cocoa.NSEraDatePickerElementFlag
            else:
                flags ^= Cocoa.NSEraDatePickerElementFlag

        self.datePickerControl.setDatePickerElements_(flags)

        self.updateControls()  # force update of all UI elements and the picker itself

    @objc.IBAction
    def setTimeElementFlags_(self, sender):
        tag = sender.selectedCell().tag()
        flags = self.datePickerControl.datePickerElements()

        checked = sender.selectedCell().state()

        if tag == kNSHourMinuteDatePickerElementFlag:
            if checked:
                flags |= Cocoa.NSHourMinuteDatePickerElementFlag
            else:
                flags ^= Cocoa.NSHourMinuteDatePickerElementFlag

        elif tag == kNSHourMinuteSecondDatePickerElementFlag:
            if checked:
                flags |= Cocoa.NSHourMinuteSecondDatePickerElementFlag
            else:
                flags ^= Cocoa.NSHourMinuteSecondDatePickerElementFlag

        elif tag == kNSTimeZoneDatePickerElementFlag:
            if checked:
                flags |= Cocoa.NSTimeZoneDatePickerElementFlag
            else:
                flags ^= Cocoa.NSTimeZoneDatePickerElementFlag

        self.datePickerControl.setDatePickerElements_(flags)

        self.updateControls()  # force update of all UI elements and the picker itself

    def updateDateTimeElementFlags(self):
        elementFlags = self.datePickerControl.datePickerElements()

        # time elements
        if (elementFlags & Cocoa.NSHourMinuteDatePickerElementFlag) != 0:
            self.timeElementChecks.selectCellWithTag_(0)
        if (elementFlags & Cocoa.NSHourMinuteSecondDatePickerElementFlag) != 0:
            self.timeElementChecks.selectCellWithTag_(1)
        if (elementFlags & Cocoa.NSTimeZoneDatePickerElementFlag) != 0:
            self.timeElementChecks.selectCellWithTag_(2)

        # date elements
        if (elementFlags & Cocoa.NSYearMonthDatePickerElementFlag) != 0:
            self.dateElementChecks.selectCellWithTag_(0)
        if (elementFlags & Cocoa.NSYearMonthDayDatePickerElementFlag) != 0:
            self.dateElementChecks.selectCellWithTag_(1)
        if (elementFlags & Cocoa.NSEraDatePickerElementFlag) != 0:
            self.dateElementChecks.selectCellWithTag_(2)

    @objc.IBAction
    def setMinDate_(self, sender):
        self.datePickerControl.setMinDate_(self.minDatePicker.dateValue())
        self.setMinDateButton.setEnabled_(False)
        self.clearMinDateButton.setEnabled_(True)

    @objc.IBAction
    def setMaxDate_(self, sender):
        self.datePickerControl.setMaxDate_(self.maxDatePicker.dateValue())
        self.setMaxDateButton.setEnabled_(False)
        self.clearMaxDateButton.setEnabled_(True)

    @objc.IBAction
    def clearMinDate_(self, sender):
        self.datePickerControl.setMinDate_(Cocoa.NSDate.distantPast())
        self.clearMinDateButton.setEnabled_(False)
        self.setMinDateButton.setEnabled_(True)

    @objc.IBAction
    def clearMaxDate_(self, sender):
        self.datePickerControl.setMaxDate_(Cocoa.NSDate.distantFuture())
        self.clearMaxDateButton.setEnabled_(False)
        self.setMaxDateButton.setEnabled_(True)

    @objc.IBAction
    def setDatePickerMode_(self, sender):
        tag = sender.selectedCell().tag()

        if tag == kSingleDateMode:
            self.datePickerControl.setDatePickerMode_(Cocoa.NSSingleDateMode)

        elif tag == kRangeDateMode:
            self.datePickerControl.setDatePickerMode_(Cocoa.NSRangeDateMode)

        self.updateControls()  # force update of all UI elements and the picker itself

    def updateDatePickerMode(self):
        mode = self.datePickerControl.datePickerMode()
        if mode == Cocoa.NSSingleDateMode:
            self.datePickerModeRadios.selectCellWithTag_(0)

            # interval value not applicable:
            self.secondsRangeEdit.setEnabled_(False)
            self.secondsRangeEditLabel.setTextColor_(Cocoa.NSColor.lightGrayColor())

            self.datePickerControl.setTimeInterval_(0)

        elif mode == Cocoa.NSRangeDateMode:
            self.datePickerModeRadios.selectCellWithTag_(1)

            # interval value applies:
            self.secondsRangeEdit.setEnabled_(True)
            self.secondsRangeEditLabel.setTextColor_(Cocoa.NSColor.blackColor())

            # set the date range by start date (here we use the current date in the date picker
            # control), and time interval (in seconds)
            secsStr = self.secondsRangeEdit.stringValue()
            numSeconds = int(secsStr)
            self.datePickerControl.setTimeInterval_(numSeconds)

    def controlTextDidEndEditing_(self, notification):
        self.updateDatePickerMode()  # force update of the date picker control



import Cocoa
import objc
from fieldMath import degToRad, radToDeg


# ____________________________________________________________
class CGraphController(Cocoa.NSObject):
    graphModel = objc.IBOutlet()
    graphView = objc.IBOutlet()
    fieldNormalizeCheck = objc.IBOutlet()
    settingDrawer = objc.IBOutlet()
    fieldSlider0 = objc.IBOutlet()
    fieldSlider1 = objc.IBOutlet()
    fieldSlider2 = objc.IBOutlet()
    phaseSlider0 = objc.IBOutlet()
    phaseSlider1 = objc.IBOutlet()
    phaseSlider2 = objc.IBOutlet()
    spacingSlider = objc.IBOutlet()
    fieldDisplay0 = objc.IBOutlet()
    fieldDisplay1 = objc.IBOutlet()
    fieldDisplay2 = objc.IBOutlet()
    phaseDisplay0 = objc.IBOutlet()
    phaseDisplay1 = objc.IBOutlet()
    phaseDisplay2 = objc.IBOutlet()
    RMSGainDisplay = objc.IBOutlet()
    spacingDisplay = objc.IBOutlet()

    # ____________________________________________________________
    # Update GUI display and control values

    def awakeFromNib(self):
        self.mapImage = Cocoa.NSImage.imageNamed_("Map")
        self.graphView.setMapImage(self.mapImage)
        self.drawGraph()

    def drawGraph(self):
        self.spacingDisplay.setFloatValue_(radToDeg(self.graphModel.getSpacing()))
        self.spacingSlider.setFloatValue_(radToDeg(self.graphModel.getSpacing()))
        self.fieldDisplay0.setFloatValue_(self.graphModel.getField(0))
        self.fieldDisplay1.setFloatValue_(self.graphModel.getField(1))
        self.fieldDisplay2.setFloatValue_(self.graphModel.getField(2))
        self.fieldSlider0.setFloatValue_(self.graphModel.getField(0))
        self.fieldSlider1.setFloatValue_(self.graphModel.getField(1))
        self.fieldSlider2.setFloatValue_(self.graphModel.getField(2))
        self.phaseDisplay0.setFloatValue_(radToDeg(self.graphModel.getPhase(0)))
        self.phaseDisplay1.setFloatValue_(radToDeg(self.graphModel.getPhase(1)))
        self.phaseDisplay2.setFloatValue_(radToDeg(self.graphModel.getPhase(2)))
        self.phaseSlider0.setFloatValue_(radToDeg(self.graphModel.getPhase(0)))
        self.phaseSlider1.setFloatValue_(radToDeg(self.graphModel.getPhase(1)))
        self.phaseSlider2.setFloatValue_(radToDeg(self.graphModel.getPhase(2)))

        totalField = (
            self.graphModel.getField(0)
            + self.graphModel.getField(1)
            + self.graphModel.getField(2)
        )

        RMSGain = self.graphModel.fieldGain()
        self.graphView.setGain(RMSGain, totalField)
        self.RMSGainDisplay.setFloatValue_(RMSGain * 100.0)

        path, maxMag = self.graphModel.getGraph()
        self.graphView.setPath(path, maxMag)

    # ____________________________________________________________
    # Handle GUI values

    @objc.IBAction
    def fieldDisplay0_(self, sender):
        self.setNormalizedField(0, sender.floatValue())
        self.drawGraph()

    @objc.IBAction
    def fieldDisplay1_(self, sender):
        self.setNormalizedField(1, sender.floatValue())
        self.drawGraph()

    @objc.IBAction
    def fieldDisplay2_(self, sender):
        self.setNormalizedField(2, sender.floatValue())
        self.drawGraph()

    @objc.IBAction
    def fieldSlider0_(self, sender):
        self.setNormalizedField(0, sender.floatValue())
        self.drawGraph()

    @objc.IBAction
    def fieldSlider1_(self, sender):
        self.setNormalizedField(1, sender.floatValue())
        self.drawGraph()

    @objc.IBAction
    def fieldSlider2_(self, sender):
        self.setNormalizedField(2, sender.floatValue())
        self.drawGraph()

    @objc.python_method
    def setNormalizedField(self, t, v):
        if self.fieldNormalizeCheck.intValue():
            f = [0, 0, 0]
            cft = 0
            for i in range(3):
                f[i] = self.graphModel.getField(i)
                cft += f[i]

            aft = cft - v
            if aft < 0.001:
                v = cft - 0.001
                aft = 0.001
            f[t] = v

            nft = 0
            for i in range(3):
                nft += f[i]
            r = aft / (nft - f[t])

            for i in range(3):
                self.graphModel.setField(i, f[i] * r)
            self.graphModel.setField(t, v)

        else:
            self.graphModel.setField(t, v)

    @objc.IBAction
    def phaseDisplay0_(self, sender):
        self.graphModel.setPhase(0, degToRad(sender.floatValue()))
        self.drawGraph()

    @objc.IBAction
    def phaseDisplay1_(self, sender):
        self.graphModel.setPhase(1, degToRad(sender.floatValue()))
        self.drawGraph()

    @objc.IBAction
    def phaseDisplay2_(self, sender):
        self.graphModel.setPhase(2, degToRad(sender.floatValue()))
        self.drawGraph()

    @objc.IBAction
    def phaseSlider0_(self, sender):
        self.graphModel.setPhase(0, degToRad(sender.floatValue()))
        self.drawGraph()

    @objc.IBAction
    def phaseSlider1_(self, sender):
        self.graphModel.setPhase(1, degToRad(sender.floatValue()))
        self.drawGraph()

    @objc.IBAction
    def phaseSlider2_(self, sender):
        self.graphModel.setPhase(2, degToRad(sender.floatValue()))
        self.drawGraph()

    @objc.IBAction
    def spacingDisplay_(self, sender):
        self.graphModel.setSpacing(degToRad(sender.floatValue()))
        self.drawGraph()

    @objc.IBAction
    def spacingSlider_(self, sender):
        self.graphModel.setSpacing(degToRad(sender.floatValue()))
        self.drawGraph()

    @objc.IBAction
    def settingDrawerButton_(self, sender):
        self.settingDrawer.toggle_(self)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSOperation(TestCase):
    def testConstants(self):
        self.assertEqual(Foundation.NSOperationQueuePriorityVeryLow, -8)
        self.assertEqual(Foundation.NSOperationQueuePriorityLow, -4)
        self.assertEqual(Foundation.NSOperationQueuePriorityNormal, 0)
        self.assertEqual(Foundation.NSOperationQueuePriorityHigh, 4)
        self.assertEqual(Foundation.NSOperationQueuePriorityVeryHigh, 8)

        self.assertIsInstance(Foundation.NSInvocationOperationVoidResultException, str)
        self.assertIsInstance(Foundation.NSInvocationOperationCancelledException, str)
        self.assertEqual(
            Foundation.NSOperationQueueDefaultMaxConcurrentOperationCount, -1
        )

        self.assertEqual(
            Foundation.NSOperationQualityOfServiceUserInteractive,
            Foundation.NSQualityOfServiceUserInteractive,
        )
        self.assertEqual(
            Foundation.NSOperationQualityOfServiceUserInitiated,
            Foundation.NSQualityOfServiceUserInitiated,
        )
        self.assertEqual(
            Foundation.NSOperationQualityOfServiceUtility,
            Foundation.NSQualityOfServiceUtility,
        )
        self.assertEqual(
            Foundation.NSOperationQualityOfServiceBackground,
            Foundation.NSQualityOfServiceBackground,
        )

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSOperation.isCancelled)
        self.assertResultIsBOOL(Foundation.NSOperation.isExecuting)
        self.assertResultIsBOOL(Foundation.NSOperation.isFinished)
        self.assertResultIsBOOL(Foundation.NSOperation.isConcurrent)
        self.assertResultIsBOOL(Foundation.NSOperation.isReady)

        self.assertResultIsBOOL(Foundation.NSOperationQueue.isSuspended)
        self.assertArgIsBOOL(Foundation.NSOperationQueue.setSuspended_, 0)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBlock(Foundation.NSOperation.completionBlock, b"v")
        self.assertArgIsBlock(Foundation.NSOperation.setCompletionBlock_, 0, b"v")

        self.assertArgIsBlock(
            Foundation.NSBlockOperation.blockOperationWithBlock_, 0, b"v"
        )
        self.assertArgIsBlock(Foundation.NSBlockOperation.addExecutionBlock_, 0, b"v")

        self.assertArgIsBOOL(
            Foundation.NSOperationQueue.addOperations_waitUntilFinished_, 1
        )
        self.assertArgIsBlock(
            Foundation.NSOperationQueue.addOperationWithBlock_, 0, b"v"
        )

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(Foundation.NSOperation.isAsynchronous)

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertArgIsBlock(Foundation.NSOperationQueue.addBarrierBlock_, 0, b"v")
